#include "allocv.h"
#include "seplib.h"
#include "dot_prodf.h"
#include <vector>
#include "cgstep.h"
#include <numeric>
#include <algorithm>
#include <functional>
#include <iostream>

int Step::cgstep(int &forget,std::vector<float> &x,
                             std::vector<float> &g,
                             std::vector<float> &R,
                             std::vector<float> &G) {
    int fcgstep;
    double SdS, GdG, GdS, determ, GdR, SdR, alpha, beta;

  if(s.size() == 0) {
    forget=1;
    allocV(s,x.size());
    allocV(S,R.size());
  }

  if(forget == 1) {
    for(int jj=0; jj<s.size(); jj++) {
      s[jj]=0.0;
    }
    for(int jj=0; jj<S.size(); jj++) {
      S[jj]=0.0;
    }
    beta=0;
    if(dot_prodf(G,G) == 0.)
      seperr("cgstep: grad vanishes identically");
    alpha = -dot_prodf(G,R)/dot_prodf(G,G);
  }
  else {
    GdG = dot_prodf(G,G);
    SdS = dot_prodf(S,S);
  
    // G.(R-G*alpha-S*beta)=0
    GdS = dot_prodf(G,S);
  
    // G.(R-G*alpha-S*beta)=0
    if(GdG == 0 || SdS ==0) {
      fcgstep=1;
      return fcgstep;
    }
  
    determ = GdG*SdS*std::max(1.-(GdS/GdG)*(GdS/SdS),1.0e-12);
    GdR = -dot_prodf(G,R);
    SdR = -dot_prodf(S,R);
    alpha=( SdS*GdR-GdS*SdR)/determ;
    beta= (-GdS*GdR+GdG*SdR)/determ;
  }

  // Update solution step
  update_sol(alpha,beta,g,s);

  // Update residual step
  update_sol(alpha,beta,G,S);

  // Update solution
  std::transform(x.begin(),x.end(),s.begin(),
                 x.begin(),std::plus<float>());

  // Update residual
  std::transform(R.begin(),R.end(),S.begin(),
                 R.begin(),std::plus<float>());

std::cout << "determ: " << determ << std::endl;
std::cout << "alpha: " << alpha << std::endl;
std::cout << "beta: " << beta << std::endl;
std::cout << "GdS " << GdS << std::endl;
std::cout << "GdR " << GdR << std::endl;
std::cout << "GdG " << GdG << std::endl;
std::cout << "SdS " << SdS << std::endl;
std::cout << "SdR " << SdR << std::endl;
std::cout << "forget: " << forget << std::endl;
  forget = 0;
  fcgstep = 0;

  return fcgstep;
}

void Step::update_sol(double alpha,double beta,
                      std::vector<float> &g,std::vector<float> &s) {
// This function tries to maintain the accuracy as much as possible
  for(int jj=0; jj<s.size(); jj++) {
    double temp1=alpha*g[jj];
    double temp2=beta *s[jj];
    s[jj]=temp1+temp2;
  }


}
