#include "read.h"
#include "Obj/props.h"
#include <vector>
#include "lap.h"
#include "alloc.h"
#include "allocv.h"
#include <iostream>
//#include <tbb/tbb.h>
//using namespace tbb;
//using namespace ispc;

void Lap::lap_init(Read par) {
  // Some shortcuts
  nkgx=par.nkgx;
  nkz =par.nkz;
  nkgxEx=nkgx+2*buff;
  nkzEx =nkz +2*buff;

  // Coefficients of 2nd deriv., 10th order approx.
  // Courtesy of Ettore Biondi
  c0 = -2.927222222222182   ; 
  c1 =  1.666666666666642   ;
  c2 = -0.238095238095235   ;
  c3 =  0.039682539682539   ;
  c4 = -0.004960317460317   ;
  c5 =  3.174603174603100e-4;

  // Create coefficients with the spacings 
  float idx=1.0/par.dx;
  float idz=1.0/par.dz;
  float idt2=par.dt*par.dt;
  a0x=c0*idx*idx*idt2;
  a1x=c1*idx*idx*idt2;
  a2x=c2*idx*idx*idt2;
  a3x=c3*idx*idx*idt2;
  a4x=c4*idx*idx*idt2;
  a5x=c5*idx*idx*idt2;
                
  a0z=c0*idz*idz*idt2;
  a1z=c1*idz*idz*idt2;
  a2z=c2*idz*idz*idt2;
  a3z=c3*idz*idz*idt2;
  a4z=c4*idz*idz*idt2;
  a5z=c5*idz*idz*idt2;

  a0=a0x+a0z;

  // Allocate auxiliary array
   allocV(dpcur,nkgxEx*nkzEx);  
}

void Lap::lap(std::vector<float>&pold,
              std::vector<float>&pcur,
              std::vector<float>&pnew,
              std::vector<float> &Vel2,int adj) {
  // Update the wavefield frames
  pold.swap(pcur);
  pcur.swap(pnew);

  if(adj==1) {
    // Multiply pcur by Vel2 before Laplacian computation
    #pragma omp parallel for
    for(int jgx=buff; jgx<nkgx+buff; jgx++) {
      #pragma omp simd simdlen(8)
      for(int jz=buff; jz<nkz+buff; jz++) { 
        int jj=jz+jgx*nkzEx;
        int ii=jz-buff+(jgx-buff)*nkz;
        dpcur[jj]=pcur[ii]*Vel2[ii];
      }
    }

    // Compute the second derivative
    #pragma omp parallel for
    for(int jgx=buff; jgx<nkgx+buff; jgx++) {
      #pragma omp simd simdlen(8)
      for(int jz=buff; jz<nkz+buff; jz++) {
        int jj=jz+jgx*nkzEx;
        int ii=jz-buff+(jgx-buff)*nkz;
        pnew[ii] = 
          a0 * dpcur[jj]+
          a1z*(dpcur[jj+1]      +dpcur[jj-1])+
          a2z*(dpcur[jj+2]      +dpcur[jj-2])+
          a3z*(dpcur[jj+3]      +dpcur[jj-3])+
          a4z*(dpcur[jj+4]      +dpcur[jj-4])+
          a5z*(dpcur[jj+5]      +dpcur[jj-5])+
          a1x*(dpcur[jj+1*nkzEx]+dpcur[jj-1*nkzEx])+
          a2x*(dpcur[jj+2*nkzEx]+dpcur[jj-2*nkzEx])+
          a3x*(dpcur[jj+3*nkzEx]+dpcur[jj-3*nkzEx])+
          a4x*(dpcur[jj+4*nkzEx]+dpcur[jj-4*nkzEx])+
          a5x*(dpcur[jj+5*nkzEx]+dpcur[jj-5*nkzEx])+2.*pcur[ii]-pold[ii];
      }
    }
  }
  else {
    #pragma omp parallel for
    for(int jgx=buff; jgx<nkgx+buff; jgx++) {
      #pragma omp simd simdlen(8)
      for(int jz=buff; jz<nkz+buff; jz++) {
        int jj=jz+jgx*nkzEx;
        int ii=jz-buff+(jgx-buff)*nkz;
        dpcur[jj]=pcur[ii];
      }
    }
    // Compute the second derivative
    #pragma omp parallel for
    for(int jgx=buff; jgx<nkgx+buff; jgx++) {
      #pragma omp simd simdlen(8)
      for(int jz=buff; jz<nkz+buff; jz++) {
        int jj=jz+jgx*nkzEx;
        int ii=jz-buff+(jgx-buff)*nkz;
        pnew[ii] = Vel2[ii]*( 
          a0 * dpcur[jj]+
          a1z*(dpcur[jj+1]      +dpcur[jj-1])+
          a2z*(dpcur[jj+2]      +dpcur[jj-2])+
          a3z*(dpcur[jj+3]      +dpcur[jj-3])+
          a4z*(dpcur[jj+4]      +dpcur[jj-4])+
          a5z*(dpcur[jj+5]      +dpcur[jj-5])+
          a1x*(dpcur[jj+1*nkzEx]+dpcur[jj-1*nkzEx])+
          a2x*(dpcur[jj+2*nkzEx]+dpcur[jj-2*nkzEx])+
          a3x*(dpcur[jj+3*nkzEx]+dpcur[jj-3*nkzEx])+
          a4x*(dpcur[jj+4*nkzEx]+dpcur[jj-4*nkzEx])+
          a5x*(dpcur[jj+5*nkzEx]+dpcur[jj-5*nkzEx]))+2.*pcur[ii]-pold[ii];
      }
    }
  }
}

//  tbb::parallel_for(buff,nkgx+buff,[&](int jgx) {
//    props(pcur.data(),pnew.data(),nkz,jgx,dx,dz,buff);
//  });

