#include "read.h"
#include "write.h"
#include "lap.h"
#include "grad.h"
#include "vshot.h"
#include "rshot.h"
#include "lint2.h"
#include "wave_prop.h"
#include "scatter.h"
#include "scale.h"
#include "wemva.h"
#include "operator.h"
#include "hess_op.h"
#include "dso.h"
#include "wemva_op.h"
#include "lwivu_op.h"
#include <vector>
#include <iterator>
#include <iostream>
#include <algorithm>
#include <functional>

Lwivu_op::Lwivu_op(Read &par_in,Hess_op &hess_op_in,
                   Wemva_op &wemva_op_in, Dso &dso_in,int doit) {
  if(doit == 1) {
    par=&par_in;
    hess_op=&hess_op_in;
    wemva_op=&wemva_op_in;
    dso=&dso_in;
    nt=(*par).nt;
    nkz=(*par).nkz;
    nkgx=(*par).nkgx;
    nz=(*par).nz;
    nx=(*par).nx;
    nzE=(*par).nzE;
    nxE=(*par).nxE;
    nh=(*par).nh;
    lambda=(*par).lambda;
    n1=nxE*(2*nh+1)*nzE;
    n2=nkgx*(2*nh+1)*nkz;
  }
}

Lwivu_op::~Lwivu_op() {}

void Lwivu_op::op(int adj,int add,
                      std::vector<float> &model,
                      std::vector<float> &data) {
  std::vector<float> data_temp;
  std::vector<float> model_temp;

  if(adj == 1) {
    if(add == 0) {
//      #pragma omp parallel for
      for(int jj=0; jj<model.size(); jj++) {
        model[jj] = 0.0;
      }
    }

    std::vector<float> model1(n1,0.);
    std::vector<float> model2(n2,0.);
    std::vector<float> data1;
    std::vector<float> data2;

    // Split the data input
    std::vector<float>::iterator it1, it2;
    it1=data.begin()+n1-1;
    it2=data.begin()+n1;

    data1.insert(data1.begin(),data.begin(),it1);
    data2.insert(data2.begin(),it2,data.end());

    // Compute -lambda^2*D*data2
    std::transform(data2.begin(),data2.end(),data2.begin(),
    std::bind1st(std::multiplies<float>(),-lambda*lambda));
//    data_temp.insert(data_temp.begin(),data2.begin(),data2.end());
//    (*dso).op(1,0,data2,data_temp);
//    data_temp.clear();

    // Sum the data: data2=-data1+lambda*D*data2
    std::transform(data2.begin(),data2.end(),data1.begin(),
                   data2.begin(),std::minus<float>());

    // Adjoint modeling
    (*hess_op).op (1,add,model1,data1);
    (*wemva_op).op(1,add,model2,data2);

    // Insert into model_temp
    model_temp.insert(model_temp.end(),model1.begin(),model1.end());
    model_temp.insert(model_temp.end(),model2.begin(),model2.end());

    for(int jj=0; jj<model.size(); jj++) {
      model[jj]+=model_temp[jj];
    }
  }
  else {
    if(add == 0) {
//      #pragma omp parallel for
      for(int jj=0; jj<data.size(); jj++) {
        data[jj] = 0.0;
      }
    }

    std::vector<float> data1(n1,0.);
    std::vector<float> data2(n2,0.);
    std::vector<float> model1;
    std::vector<float> model2;

    // Split the model input
    std::vector<float>::iterator it1,it2;
    it1=model.begin()+n1-1;
    it2=model.begin()+n1;

    model1.insert(model1.begin(),model.begin(),it1);
    model2.insert(model2.begin(),it2,model.end());

    // Forward modeling
    (*hess_op).op (0,add,model1,data1);
    (*wemva_op).op(0,add,model2,data2);

    // Sum the computed data: data1-data2
//    std::transform(data2.begin(),data2.end(),data2.begin(),
//    std::bind1st(std::multiplies<float>(),-1.));
    std::transform(data1.begin(),data1.end(),data2.begin(),
                   data1.begin(),std::minus<float>());

//    // Compute D*data2
//    data_temp.insert(data_temp.begin(),data2.begin(),data2.end());
//    (*dso).op(0,0,data_temp,data2); 
//    data_temp.clear();
 
    // Insert into data_temp
    data_temp.insert(data_temp.end(),data1.begin(),data1.end());
    data_temp.insert(data_temp.end(),data2.begin(),data2.end());

    for(int jj=0; jj<data.size(); jj++) {
      data[jj]+=data_temp[jj];
    }
  }
}

