#include "read.h"
#include "write.h"
#include "lap.h"
#include "grad.h"
#include "vshot.h"
#include "rshot.h"
#include "lint2.h"
#include "wave_prop.h"
#include "scatter.h"
#include "scale.h"
#include "wemva.h"
#include "operator.h"
#include "hess_op.h"
#include "wemva_op.h"
#include "lwivu_op.h"
#include "lwivu_reg_op.h"
#include <vector>
#include <iterator>
#include <iostream>
#include <algorithm>
#include <functional>

Lwivu_reg_op::Lwivu_reg_op(Read &par_in,Wemva_op &wemva_op_in,int doit) {
  if(doit == 1) {
    par=&par_in;
    wemva_op=&wemva_op_in;
    nkz=(*par).nkz;
    nkgx=(*par).nkgx;
    nh=(*par).nh;
    n1=nkgx*(2*nh+1)*nkz;
    n2=nkgx*(2*nh+1)*nkz;
  }
}

Lwivu_reg_op::~Lwivu_reg_op() {}

void Lwivu_reg_op::op(int adj,int add,
                      std::vector<float> &model,
                      std::vector<float> &data) {
  std::vector<float> data_temp;
  std::vector<float> model_temp;
  std::vector<float> zero(n1,0.);

  if(adj == 1) {
    if(add == 0) {
        model[jj] = 0.0;
      }
    }

    // Adjoint modeling
    (*wemva_op).op(1,add,model,data);

  else  {
    if(add == 0) {
      for(int jj=0; jj<data.size(); jj++) {
        data[jj] = 0.0;
      }
    }

    std::vector<float> data1(n1,0.);
    std::vector<float> data2(n2,0.);
    std::vector<float> model1;
    std::vector<float> model2;

    // Split the model input
    std::vector<float>::iterator it1,it2;
    it1=model.begin()+n1-1;
    it2=model.begin()+n1;

    model1.insert(model1.begin(),model.begin(),it1);
    model2.insert(model2.begin(),it2,model.end());

    // Forward modeling (fwd wemva result is passed)
    data1=zero;
    data2=model2;

    // Insert into data_temp
    data_temp.insert(data_temp.end(),data1.begin(),data1.end());

    // Insert into data_temp
    data_temp.insert(data_temp.end(),data2.begin(),data2.end());

  }

}
