#include "read.h"
#include "write.h"
#include "operator.h"
#include "matmult.h"
#include "tconv.h"
#include "solver_wjreg.h"
//#include "solver_cgm.h"
#include "cgstep.h"
#include <vector>
#include <sep.main>
#include <iostream>
#include <iomanip>
#include <chrono>
#include <functional>
#include <numeric>

int MAIN() {

  Solver solver; 
//  SolverCgm solver; 
  Conv tconv, tgain; 
  Step cgstep;
  Read par;
  Write out;

  int nm;
  int nd;
  int niter;
  if(0 == getch("nm","d",&nm))
    nm=10;
  if(0 == getch("nd","d",&nd))
    nd=10;
  if(0 == getch("niter","d",&niter))
    niter=nm;
  par.niter=niter;


  float rand1;
  float eps=0;
  int deriv=0;
  int nreboot=niter+1;

  std::vector<float> model(nm,0.);
  std::vector<float> data(nd,0.);
  std::vector<float> B(nm*nd,0.);
  for(int jj=0; jj<B.size(); jj++) {
    rand1=((float) rand()/(RAND_MAX));
    rand1=2.0*(rand1-0.5);
    B[jj]=rand1; 
  }

  Matmult op(B,1);

  // Pointer to the operator
  void (Op::*matmult_op)(int,int,
                 std::vector<float>&,
                 std::vector<float>&)=&Op::op;

  int (Step::*pcgstep)(int&,std::vector<float>&,
                            std::vector<float>&,
                            std::vector<float>&,
                            std::vector<float>&)=&Step::cgstep;

  // Initialize regularization filter
  int (Conv::*ptconv)(int,int,
                     std::vector<float>&,
                     std::vector<float>&)=&Conv::tconv;

  // Penalty function
  int nreg=0;
  int a=1;
  if(eps != 0) { // Use minimum norm or derivative 
    if(deriv == 1) a=-1;
    std::vector<float> bc(1,a);
    // deriv=0 min norm; deriv=1 1st derivative
    if(deriv ==1 ) bc.push_back(1);
    tconv.tconv_init(bc);
    nreg=model.size()+bc.size();
  }

  // Initialize weighting function
  int (Conv::*ptgain)(int,int,
                     std::vector<float>&,
                     std::vector<float>&)=&Conv::tconv;

  // Weighting function (I still need to find how to implement it)
//  int ntgain=par.nt;
  int ntgain=0;
  std::vector<float> tg(ntgain,0.);

  // Pass the input data to the data vector 
  for(int jj=0; jj<data.size(); jj++) {
    rand1=((float) rand()/(RAND_MAX));
    rand1=2.0*(rand1-0.5);
    data[jj]=rand1; 
  }

  // A priory model, if any
  std::vector<float> m0;

  // Preconditioner, if any (diag. of GN Hessian)
  std::vector<float> H(nm);
  std::vector<float> P(nm);
  P.clear();
  H.clear();

  // Vector for the residual
  std::vector<float> resd(data.size());

  // Vector for the residual error
  std::vector<float> err_res(niter,0.0);

  // Perform inversion
  std::cout << "Begin linear solver" << std::endl;
  solver.solver_wjreg(model,data,matmult_op,op,
                        ptconv,tconv,pcgstep,cgstep,
                        ptgain,tgain,m0,P,nreg,ntgain,
                        niter,eps,nreboot,
                        resd,err_res); 
//  solver.solver_cgm(model,data,born_op,op,
//                    ptconv,tconv,m0,P,nreg,
//                    par.niter,par.eps,
//                    resd,err_res); 
  std::cout << "Finish linear solver" << std::endl;

  // Retrieve the solution from the model vector

  // Construct the images

  std::cout << "*********** MAIN CALCULATION FINISHES! **********" << std::endl;

  // Initialize the output
  out.write_init(par);
//  
//  // Write the output
//  std::cout << "Write the inverted reflectivity" << std::endl;  
//  out.write_image(par,rtm.ICorr.data(),"rtm_cic");
//
//  // Write the residual 
//  std::cout << "Write the residual binary" << std::endl;  
//  float **D;
//  alloc(D,par.nkgx,par.nt);
//  for(int jgx=0; jgx<par.nkgx; jgx++) {
//    for(int jt=0; jt<par.nt; jt++) {
//      int jj=jt+jgx*par.nt;
//      D[jgx][jt] = resd[jj];
//    }
//  }
//  out.write_data(par,D,"resid");
//  delete D;

  // Write the residual error for plotting
  if(!err_res.empty()) out.write_error_res(par,niter,err_res.data(),"error_res");
  hclose();
  return 0;
}
