#include "read.h"
#include "vshot.h"
#include "minmax.h"
#include <iostream>
#include <algorithm>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cmath>

void VShot::vshot_init(Read par) {
  // Shorcuts for frequently object's members invokation
  pad =par.pad;
  zpad=par.zpad;
  nbox=par.nbox;
  nkgx=par.nkgx;
  nkz =par.nkz;
  nx  =par.nx;
  nz  =par.nz;
  nt  =par.nt;
  napL=par.napL;
  napR=par.napR;
  dt  =par.dt;
  dx  =par.dx;
  dz  =par.dz;
  first_cdp=par.first_cdp;
  last_cdp=par.last_cdp;
  src_pad=par.src_pad;
  upad=pad+zpad;

  // Initialize distances for the definition of the RBCs
  distZU=new float[pad];
  distZD=new float[pad];
  distXL=new float[pad];
  distXR=new float[pad];
  distU =new float[pad];
  distD =new float[pad];
  distL =new float[pad];
  distR =new float[pad];

  // This factor balances the difference in the spacings.
  // rand_coeff regulates the "agressivity" of randomness.
  fact = dx/dz;
  if(fact < 1.0)
    fact = par.rand_coeff/fact;
  else
    fact = par.rand_coeff*fact;

  for(int jj=0; jj<pad; jj++)
    distZU[jj] = fact*dz*(pad-jj-1);

  for(int jj=0; jj<pad; jj++)
    distZD[jj] = fact*dz*jj;

  for(int jj=0; jj<pad; jj++)
    distXL[jj] = fact*dx*(pad-jj-1);

  for(int jj=0; jj<pad; jj++)
    distXR[jj] = fact*dx*jj;

  distMax=pad-1;
  distMaxU=pad-1;
  for(int jj=0; jj<pad; jj++) 
    distU[jj] = 1.0*(pad-1-jj)/distMaxU;

  for(int jj=0; jj<pad; jj++) 
    distL[jj] = 1.0*(pad-1-jj)/distMax;

  for(int jj=0; jj<pad; jj++) 
    distD[jj]  = 1.0*jj/distMax;

  for(int jj=0; jj<pad; jj++) 
    distR[jj]  = 1.0*jj/distMax;
}

void VShot::vshot(Read par,float **Vint,float **Vshot,float *sgx,int rnd) {
  // Obtain minimum and maximum values of velocities
  Vmin=minval(Vint,nz,nx);
  if(Vmin > 0.0) // In case Vmin < 0 keep that value (when using
    Vmin=0;      // reflectivity instead of velocity).
  Vmax=maxval(Vint,nz,nx);

  // Initialize velocity frame
//  #pragma omp parallel for
  for(int jgx=0; jgx<nkgx; jgx++) {
//    #pragma omp simd
    for(int jz=0; jz<nkz; jz++)
      Vshot[jgx][jz]=0.0;
  }

  if(rnd==1) srand(time(NULL));
  damp1=1;
  damp2=1;
  frac=1.0;

  l=0;
  for(int jgx=0; jgx<nbox+2*pad+src_pad; jgx++) {
    if(jgx >= pad && jgx < nbox+pad+src_pad ) {
      jx=1+floor(sgx[jgx-pad]/dx);
      if(jx >= first_cdp && jx <= last_cdp) {
        for(int jz=upad; jz<upad+nz; jz++)
          Vshot[jgx][jz]=Vint[jx-first_cdp][jz-upad];
        for(int jz=pad; jz<upad; jz++)
          Vshot[jgx][jz]=Vint[jx-first_cdp][0];
      
        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              damp1=sin(0.5*PI*damp1);
            }
            Vtest=Vint[jx-first_cdp][0]*damp1+rand1*distZU[jz];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }
      
        k=0; 
        for(int jz=nz+upad; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              damp1=sin(0.5*PI*damp1);
            }
            Vtest=Vint[jx-first_cdp][nz-1]*damp1+rand1*distZD[k];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }
      else if(jx < first_cdp) {
        for(int jz=upad; jz<upad+nz; jz++)
          Vshot[jgx][jz]=Vint[0][jz-upad];
        for(int jz=pad; jz<upad; jz++)
          Vshot[jgx][jz]=Vint[0][0];
 
        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              damp1=sin(0.5*PI*damp1);
            }
            Vtest=Vint[0][0]*damp1+rand1*distZU[jz];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }
 
        k=0; 
        for(int jz=nz+upad; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              damp1=sin(0.5*PI*damp1);
            }
            Vtest=Vint[0][nz-1]*damp1+rand1*distZD[k];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }
      else if(jx > last_cdp) {
        for(int jz=upad; jz<upad+nz; jz++) 
          Vshot[jgx][jz]=Vint[nx-1][jz-upad];
        for(int jz=0; jz<upad; jz++) 
          Vshot[jgx][jz]=Vint[nx-1][0];
 
        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              damp1=sin(0.5*PI*damp1);
            }
            Vtest=Vint[nx-1][0]*damp1+rand1*distZU[jz];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }
 
        k=0; 
        for(int jz=nz+upad; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              damp1=sin(0.5*PI*damp1);
            }
            Vtest=Vint[nx-1][nz-1]*damp1+rand1*distZD[k];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }
    }
    else if(jgx < pad) {
      jx=1+floor(sgx[0]/dx);
      if(jx>=first_cdp && jx<= last_cdp) {
        for(int jz=pad; jz<nz+upad; jz++) {
          do {
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp2=frac*(1-distL[jgx]);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp2=sin(0.5*PI*damp2);
            }
            if(jz < upad)
              Vtest=Vint[jx-first_cdp][0]*damp2+rand2*distXL[jgx];
            else
              Vtest=Vint[jx-first_cdp][jz-upad]*damp2+rand2*distXL[jgx];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              damp2=frac*(1-distL[jgx]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[jx-first_cdp][0]*damp1*damp2+rand1*distZU[jz]+rand2*distXL[jgx];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        k=0;
        for(int jz=upad+nz; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              damp2=frac*(1-distL[jgx]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[jx-first_cdp][nz-1]*damp1*damp2+rand1*distZD[k]+rand2*distXL[jgx];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }
      else if(jx < first_cdp) {
        for(int jz=pad; jz<nz+upad; jz++) {
          do {
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp2=frac*(1-distL[jgx]);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp2=sin(0.5*PI*damp2);
            }
            if(jz < upad)
              Vtest=Vint[0][0]*damp2+rand2*distXL[jgx];
            else
              Vtest=Vint[0][jz-upad]*damp2+rand2*distXL[jgx];
            if(Vtest <= Vmax && Vtest >= Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              damp2=frac*(1-distL[jgx]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[0][0]*damp1*damp2+rand1*distZU[jz]+rand2*distXL[jgx];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        k=0;
        for(int jz=nz+upad; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              damp2=frac*(1-distL[jgx]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[0][nz-1]*damp1*damp2+rand1*distZD[k]+rand2*distXL[jgx];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }
      else if(jx > last_cdp) {
        for(int jz=pad; jz<nz+upad; jz++) {
          do {
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp2=frac*(1-distL[jgx]);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp2=sin(0.5*PI*damp2);
            }
            if(jz < upad)
              Vtest=Vint[nx-1][0]*damp2+rand2*distXL[jgx];
            else
              Vtest=Vint[nx-1][jz-upad]*damp2+rand2*distXL[jgx];
            if(Vtest <= Vmax && Vtest >= Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              damp2=frac*(1-distL[jgx]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[nx-1][0]*damp1*damp2+rand1*distZU[jz]+rand2*distXL[jgx];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        k=0;
        for(int jz=nz+upad; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              damp2=frac*(1-distU[jgx]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[nx-1][nz-1]*damp1*damp2+rand1*distZD[k]+rand2*distXL[jgx];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }
    }
    else {
      jx=1+floor(sgx[nbox+src_pad-1]/dx);
      if(jx>=first_cdp && jx<= last_cdp) {
        for(int jz=pad; jz<nz+upad; jz++) {
          do {
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp2=frac*(1-distR[l]);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp2=sin(0.5*PI*damp2);
            }
            if(jz < upad)
              Vtest=Vint[jx-first_cdp][0]*damp2+rand2*distXR[l];
            else
              Vtest=Vint[jx-first_cdp][jz-upad]*damp2+rand2*distXR[l];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              damp2=frac*(1-distR[l]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[jx-first_cdp][0]*damp1*damp2+rand1*distZU[jz]+rand2*distXR[l];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        k=0;
        for(int jz=nz+upad; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              damp2=frac*(1-distR[l]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[jx-first_cdp][nz-1]*damp1*damp2+rand1*distZD[k]+rand2*distXR[l];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }
      else if(jx < first_cdp) {
        for(int jz=pad; jz<nz+upad; jz++) {
          do {
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp2=frac*(1-distR[l]);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp2=sin(0.5*PI*damp2);
            }
            if(jz < upad)
              Vtest=Vint[0][0]*damp2+rand2*distXR[l];
            else
              Vtest=Vint[0][jz-upad]*damp2+rand2*distXR[l];

            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              damp2=frac*(1-distR[l]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[0][0]*damp1*damp2+rand1*distZU[jz]+rand2*distXR[l];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        k=0;
        for(int jz=nz+upad; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              damp2=frac*(1-distR[l]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[0][nz-1]*damp1*damp2+rand1*distZD[k]+rand2*distXR[l];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }
      else if(jx > last_cdp) {
        for(int jz=pad; jz<nz+upad; jz++) {
          do {
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp2=frac*(1-distR[l]);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp2=sin(0.5*PI*damp2);
            }
            if(jz < upad)
              Vtest=Vint[nx-1][0]*damp2+rand2*distXR[l];
            else
              Vtest=Vint[nx-1][jz-upad]*damp2+rand2*distXR[l];

            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        for(int jz=0; jz<pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distU[jz]);
              damp2=frac*(1-distR[l]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[nx-1][0]*damp1*damp2+rand1*distZU[jz]+rand2*distXR[l];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
        }

        k=0;
        for(int jz=nz+upad; jz<nz+upad+pad; jz++) {
          do {
            rand1=0.0;
            damp1=1.0;
            rand2=0.0;
            damp2=1.0;
            if(rnd==1) {
              damp1=frac*(1-distD[k]);
              damp2=frac*(1-distR[l]);
              rand1=((float) rand()/(RAND_MAX));
              rand1=2.0*(rand1-0.5);
              rand2=((float) rand()/(RAND_MAX));
              rand2=2.0*(rand2-0.5);
              damp1=sin(0.5*PI*damp1);
              damp2=sin(0.5*PI*damp2);
            }
            Vtest=Vint[nx-1][nz-1]*damp1*damp2+rand1*distZD[k]+rand2*distXR[l];
            if(Vtest<=Vmax && Vtest>=Vmin) {
              Vshot[jgx][jz]=Vtest;
              break;
            }
          } while(1);
          k++;
        }
      }  
      l++;
    } 
  } // ending for over jgx
}
