/*
 This code prepares velocity or reflectivity frames within the appropriate
shot frame, including aperture and, if needed, padding. Velocity is computed
with or without random boundary conditions (RBC) according to Clapp (2009).
Henceforth in the .cpp file I'll use "velocity" in the comments for referring
to the "velocity" arrays, despite the entry can be reflectivity.
*/
#ifndef VSHOT_H
#define VSHOT_H
#include <cmath>

# define PI 3.1415926535897932 // The Order of Chaos

class VShot {
  public:
    void vshot_init(Read);
    void vshot(Read,float**,float**,float*,int);

  protected:
    int jx, k, l;
    int nt, nx, nbox, nkgx, nh, nz, nkz, napL, napR;
    int first_cdp, last_cdp, pad, zpad, upad, src_pad;
    float dt, dx, dz, fx;
    float rand1, rand2, fact, damp1, damp2, frac;
    float Vmin, Vmax, Vtest;
    float distMax, distMaxU;
    float *distXL, *distXR, *distZU, *distZD;
    float *distU, *distD, *distL, *distR;
};

#endif
