#include "read.h"
#include "operator.h"
#include "dso.h"
#include <vector>
#include <iostream>
#include <cmath>

Dso::Dso(Read &par_in,int do_dso_in) {
  par=&par_in;
  nkz=(*par).nkz;
  nkgx=(*par).nkgx;
  nh=(*par).nh;
  dh=(*par).dh;
  dx=(*par).dx;
  do_dso=do_dso_in;
}

Dso::~Dso() {}

void Dso::op(int adj,int add,
                      std::vector<float> &model,
                      std::vector<float> &data) {
  const float pi=3.14159265359;
  const float eps=0.0000001;
  float h, hmax, dso;
  hmax=nh*dh*dx;
  if(adj == 1) {
    int nxz=model.size()/(2*nh+1);
    if(add == 0) {
      #pragma omp parallel for
      for(int jj=0; jj<model.size(); jj++) {
        model[jj] = 0.0;
      }
    }
    if(do_dso == 1) {
      for(int jh=-nh; jh<=nh; jh++) {
        if(nh==0) {
          dso=1.; // If h=0, use stacking power maximization
        }
        else {
          h=jh*dh*dx;
//          dso=(pow(h,2)/2.+pow(h,4)*3./8.+pow(h,6)*5./16.)/hmax;
//          dso=std::abs(h);
          dso=1-exp(-h*h);
//          float heps=h+eps;
//          dso=hmax*(1-exp(-h*h))*sin(pi*heps/hmax)/heps;
//          float hl=h+hmax;
//          float hr=h-hmax;
//          dso=pow((1-exp(-h*h))*(1-exp(-hl*hl))*(1-exp(-hr*hr)),2);
        }
        for(int jxz=0; jxz<nxz; jxz++) {
          int jj=jxz+(jh+nh)*nxz;
          model[jj]+=data[jj]*dso;
        }
      }
    }
    else {
      for(int jj=0; jj<data.size(); jj++) {
        model[jj]+=data[jj];
      }
    }
  }
  else {
    int nxz=data.size()/(2*nh+1);
    if(add == 0) {
      #pragma omp parallel for
      for(int jj=0; jj<data.size(); jj++) {
        data[jj] = 0.0;
      }
    }
    if(do_dso == 1) {
      for(int jh=-nh; jh<=nh; jh++) {
        if(nh==0) {
          dso=1.; // If h=0, use stacking power maximization
        }
        else {
          h=jh*dh*dx;
//          dso=(pow(h,2)/2.+pow(h,4)*3./8.+pow(h,6)*5./16.)/hmax;
          dso=1-exp(-h*h);
//          dso=std::abs(h);
//          float heps=h+eps;
//          dso=hmax*(1-exp(-h*h))*sin(pi*heps/hmax)/heps;
//          float hl=h+hmax;
//          float hr=h-hmax;
//          dso=pow((1-exp(-h*h))*(1-exp(-hl*hl))*(1-exp(-hr*hr)),2);
        }
        for(int jxz=0; jxz<nxz; jxz++) {
          int jj=jxz+(jh+nh)*nxz;
          data[jj]+=model[jj]*dso;
        }
      }
    }
    else {
      for(int jj=0; jj<model.size(); jj++) {
        data[jj]+=model[jj];
      }
    }
  }
}


