#include "read.h"
#include <vector>
#include "grad2.h"
#include "allocv.h"
#include <iostream>

Grad2::Grad2(Read par) {
  // Some shortcuts
  nkgx=par.nkgx;
  nkz =par.nkz;
  nkgxEx=nkgx+2*buff;
  nkzEx =nkz +2*buff;
  dx=par.dx;
  dz=par.dz;

  // Coefficients of 2nd deriv., 10th order approx.
  // Courtesy of Ettore Biondi
  c0 = -2.927222222222182   ; 
  c1 =  1.666666666666642   ;
  c2 = -0.238095238095235   ;
  c3 =  0.039682539682539   ;
  c4 = -0.004960317460317   ;
  c5 =  3.174603174603100e-4;

  // Create coefficients with the spacings 
  float idx=1.0/par.dx;
  float idz=1.0/par.dz;
  a0x=c0*idx*idx;
  a1x=c1*idx*idx;
  a2x=c2*idx*idx;
  a3x=c3*idx*idx;
  a4x=c4*idx*idx;
  a5x=c5*idx*idx;
                
  a0z=c0*idz*idz;
  a1z=c1*idz*idz;
  a2z=c2*idz*idz;
  a3z=c3*idz*idz;
  a4z=c4*idz*idz;
  a5z=c5*idz*idz;

  a0=a0x+a0z;

  // Allocate auxiliary array
   allocV(dpcur,nkgxEx*nkzEx);  
}

void Grad2::grad2(std::vector<float>&p,
                  std::vector<float>&dpxz) {
 #pragma omp parallel for
 for(int jgx=0; jgx<nkgxEx; jgx++) {
   #pragma omp simd
   for(int jz=0; jz<nkzEx; jz++) {
     int jj=jz+jgx*nkzEx;
     dpcur[jj]=0.;
   }
 }
 #pragma omp parallel for
 for(int jgx=buff; jgx<nkgx+buff; jgx++) {
   #pragma omp simd
   for(int jz=buff; jz<nkz+buff; jz++) {
     int jj=jz+jgx*nkzEx;
     int ii=jz-buff+(jgx-buff)*nkz;
     dpcur[jj]=p[ii];
   }
 }

 // Compute the second derivative
 d2dxz(dpxz);
}

void Grad2::d2dxz(std::vector<float>&dp) {
  // Second order derivative 
  #pragma omp parallel for
  for(int jgx=buff; jgx<nkgx+buff; jgx++) {
    #pragma omp simd
    for(int jz=buff; jz<nkz+buff; jz++) {
      int jj=jz+jgx*nkzEx;
      int ii=jz-buff+(jgx-buff)*nkz;
      dp[ii] = 
        a0 * dpcur[jj]+
        a1z*(dpcur[jj+1]      +dpcur[jj-1])+
        a2z*(dpcur[jj+2]      +dpcur[jj-2])+
        a3z*(dpcur[jj+3]      +dpcur[jj-3])+
        a4z*(dpcur[jj+4]      +dpcur[jj-4])+
        a5z*(dpcur[jj+5]      +dpcur[jj-5])+
        a1x*(dpcur[jj+1*nkzEx]+dpcur[jj-1*nkzEx])+
        a2x*(dpcur[jj+2*nkzEx]+dpcur[jj-2*nkzEx])+
        a3x*(dpcur[jj+3*nkzEx]+dpcur[jj-3*nkzEx])+
        a4x*(dpcur[jj+4*nkzEx]+dpcur[jj-4*nkzEx])+
        a5x*(dpcur[jj+5*nkzEx]+dpcur[jj-5*nkzEx]);
    }
  }
}

