#include "read.h"
#include "write.h"
#include "alloc.h"
#include "allocv.h"
#include <cmath>
#include <vector>
#include "lint2.h"
#include "operator.h"
#include "interp_psf.h"
#include "hess_op.h"
#include "base_trismt.h"
#include "seplib.h"
#include "sep3d.h"
#include "sep_pars_external.h"
#include <iostream>
#include <chrono>

Hess_op::Hess_op(Read &par,InterpPsf &psf_in,int doit) {
  if(doit == 1) {
    // Shorcuts for frequently used object's members
    pad=par.pad;
    zpad=par.zpad;
    nxE=par.nxE;
    nzE=par.nzE;
    dx=par.dx;
    dz=par.dz;
    first_cdp=par.first_cdp;
    last_cdp=par.last_cdp;
    nh=par.nh;
    dh=par.dh;
    hmax=par.hmax;
    nlagx=par.nlagx;
    nlagz=par.nlagz;
    incPSFz=par.incPSFz;
    incPSFx=par.incPSFx;
    nxc=par.npsfx;
    nzc=par.npsfz;

    psf=&psf_in;

    // Allocate arrays for Hessian computation
    int n=nxE*nzE;
    allocV(Hgn,n);
  }
}

Hess_op::~Hess_op() {}

void Hess_op::op(int adj,int add,std::vector<float>&model,
                                 std::vector<float>&data) {
  std::cout << "Perform Hessian multip." << std::endl;
  int ntemp=nzE*nxE;
  std::vector<float> maux(ntemp,0.);
  if(adj==1) { // Adjoint Hessian multiplication
    if(add==0) {
      std::fill(model.begin(),model.end(),0.);
    }
    for(int jh=0; jh<2*nh+1; jh++) {
      if(4*ntemp != sreed("psf",maux.data(),4*ntemp))
        seperr("hess_op: Cannot read psf file");
      for(int jlagx=-nlagx; jlagx<=nlagx; jlagx++) { 
        for(int jlagz=-nlagz; jlagz<=nlagz; jlagz++) { 
          (*psf).op(jlagx,jlagz,maux,Hgn);
//          #pragma omp parallel for 
          for(int jx=nlagx; jx<nxE+nlagx; jx++) {
//            #pragma omp simd
            for(int jz=nlagz; jz<nzE+nlagz; jz++) {
              int jj=jz-nlagz+(jx-nlagx)*nzE;
              int j1=jz-nlagz+(jx-nlagx)*nzE+jh*nxE*nzE;
              int j2=jz+jlagz+(jx+jlagx)*(nzE+2*nlagz)+
                     jh*(nzE+2*nlagz)*(nxE+2*nlagx);
              model[j1]+=Hgn[jj]*data[j2];
            }
          }
        }
      }
    }
  }
  else { // Forward Hessian multiplication
    if(add==0) {
      std::fill(data.begin(),data.end(),0.);
    }
    for(int jh=0; jh<2*nh+1; jh++) {
      if(4*ntemp != sreed("psf",maux.data(),4*ntemp))
        seperr("hess_op: Cannot read psf file");
      for(int jlagx=-nlagx; jlagx<=nlagx; jlagx++) { 
        for(int jlagz=-nlagz; jlagz<=nlagz; jlagz++) { 
          (*psf).op(jlagx,jlagz,maux,Hgn);
//          #pragma omp parallel for collapse(2)
          for(int jx=nlagx; jx<nxE+nlagx; jx++) {
//            #pragma omp simd
            for(int jz=nlagz; jz<nzE+nlagz; jz++) {
              int jj=jz-nlagz+(jx-nlagx)*nzE;
              int j1=jz-nlagz+(jx-nlagx)*nzE+jh*nxE*nzE;
              int j2=jz+jlagz+(jx+jlagx)*(nzE+2*nlagz)+
                     jh*(nzE+2*nlagz)*(nxE+2*nlagx);
              data[j2]+=Hgn[jj]*model[j1];
            }
          }
        }
      } 
    }
  }
  auxclose("psf");
}

