#include "read.h"
#include "write.h"
#include "alloc.h"
#include "allocv.h"
#include "lint2.h"
#include "interp_psf.h"
#include "base_trismt.h"
#include "seplib.h"
#include "sep3d.h"
#include "sep_pars_external.h"
#include <vector>
#include <numeric>
#include <iostream>
#include <cmath>

InterpPsf::InterpPsf(Read &par,int doit) {
  if(doit == 1) {
    // Shorcuts for frequently used object's members
    pad=par.pad;
    zpad=par.zpad;
    nxE=par.nxE;
    nzE=par.nzE;
    n=nxE*nzE;
    nh=par.nh;
    nlagx=par.nlagx;
    nlagz=par.nlagz;
    npts=par.npts;
    dx=par.dx;
    dz=par.dz;
    first_cdp=par.first_cdp;
    last_cdp=par.last_cdp;
    dh=par.dh;
    hmax=par.hmax;
    incPSFz=par.incPSFz;
    incPSFx=par.incPSFx;

    alloc(xz,2,n);
    if(0 == auxpar("nzc","d",&nzc,"psf"))
      seperr("InterpPsf: Parameter nzc not found\n"); 
    if(0 == auxpar("nxc","d",&nxc,"psf"))
      seperr("InterpPsf: Parameter nxc not found\n"); 
    alloc(paux,nxc,nzc);
    allocV(m,nxc*nzc);
  }
}

InterpPsf::~InterpPsf() {
  delete[] paux;
}

void InterpPsf::op(int jlagx,int jlagz,std::vector<float> &maux,std::vector<float> &d) {
  // Use some shorcuts
  float d1=incPSFz*dz;
  float d2=incPSFx*dx;
  float o1=incPSFz*dz;
  float o2=(first_cdp+incPSFx-1)*dx;
  int oiz=incPSFz;
  int oix=incPSFz;

  // Extract the elements of the Hessian fron the PSFs
  int jxc=0;
  for(int jx=oix; jx<nxE; jx+=incPSFx) {
    int jzc=0;
    for(int jz=oiz; jz<nzE; jz+=incPSFz) {
      int jj=jz+jlagz+(jx+jlagx)*nzE;
      int jjc=jzc+jxc*nzc;
      m[jjc]=maux[jj];
      jzc++;
    }
    jxc++;
  }

  // Prepare coordinates for interpolation  
//  #pragma omp parallel for
  for(int jx=0; jx<nxE; jx++) {
    float x=(jx+first_cdp)*dx;
//    #pragma omp simd
    for(int jz=0; jz<nzE; jz++) {
      float z=jz*dz;
      int j=jx*nzE+jz;
      xz[0][j]=z;
      xz[1][j]=x;
    }
  }

  // Initialize 2-D linear interpolation
  lint2.lint2_init(nzc,nxc,o1,o2,d1,d2);

//  #pragma omp parallel
//  {
//  #pragma omp for
  for(int jxc=0; jxc<nxc; jxc++) {
    int j1=nzc-1+jxc*nzc;
    int j2=nzc-2+jxc*nzc;
    m[j1] = m[j2];
  }
//  #pragma omp for
  for(int jz=0; jz<nzc; jz++) {
    int j1=jz+(nxc-1)*nzc;
    int j2=jz+(nxc-2)*nzc;
    m[j1] = m[j2];
  } 
//  }

  float mean=0.;
  if(jlagx==0 && jlagz==0) {
    mean=std::accumulate(m.begin(),m.end(),0.0)/m.size();
  }

  // Interpolate psf
  for(int jx=0; jx<nxc; jx++) {
    for(int jz=0; jz<nzc; jz++) {
      int jj=jz+jx*nzc;
      if(jlagx==0 && jlagz==0 && m[jj]<=0.) {
        paux[jx][jz] = mean;
      }
      else {
        paux[jx][jz] = m[jj];
      }
    }
  }
  lint2.lint2(0,0,paux,d.data(),nzc,nxc,nxE*nzE,xz);

  // Smooth the Hessian
  if(0 != base_tris2d_oper(d,nzE,nxE,npts,npts,2)) {
    seperr("Hess_op const.: Error, triangle filter malfunction in temp");
  }
}  
