#include "read.h"
#include "lap.h"
#include "grad.h"
#include "vshot.h"
#include "rshot.h"
#include "wave_prop.h"
#include "nlmod.h"
#include "operator.h"
#include "nlmod_op.h"
#include <vector>
#include <iostream>

NLMod_op::NLMod_op(Read &par_in,NLMod &nlmod_in) {
  par=&par_in;
  nlmod=&nlmod_in;
  nt=(*par).nt;
  nkz=(*par).nkz;
  nkgx=(*par).nkgx;
  nz=(*par).nz;
  nx=(*par).nx;
  nh=(*par).nh;
  (*nlmod).nlmod_init(*par);
}

NLMod_op::~NLMod_op() {}

void NLMod_op::op(int adj,int add,
                      std::vector<float> &model,
                      std::vector<float> &data) {
  if(adj == 1) {
    if(add == 0) {
      for(int jj=0; jj<model.size(); jj++)
        model[jj] = 0.0;
    }
    for(int jgx=0; jgx<nkgx; jgx++) {
      for(int jt=0; jt<nt; jt++) {
        int j1=jt+jgx*nt;
        int j2=jt+jgx*(nt+2);
        (*par).rcv[j2]=data[j1];
      }
    }

    (*par).adj=1;
    (*nlmod).nlmod(*par,0);

    for(int jj=0; jj<model.size(); jj++) {
      model[jj] = (*par).src[jj];
    }
  }
  else {
    if(add == 0) {
      for(int jj=0; jj<data.size(); jj++) {
        data[jj] = 0.0;
      }
    }
    for(int jj=0; jj<model.size(); jj++) {
      (*par).src[jj] = model[jj];
    }

    (*par).adj=0;
    (*nlmod).nlmod(*par,0);

    for(int jj=0; jj<data.size(); jj++) {
      data[jj] = (*nlmod).dsynt[jj];
    }
  }
}

