/*
  ********************************************************************
  *** This class encompasses functions for reading different input ***
  *** parameters, both from command line and files.                ***
  ********************************************************************
*/
#include "param_mpi.h"
#include "allocv.h"
#include "seplib.h"
#include "sep3d.h"
#include "sep_pars_external.h"
#include <string>
#include <iostream>
#include <vector>
#include <cmath>
#include <string>

int ParamMPI::read_num_files(char* in_pref) {
std::cout << "read_num_files: Find the number of files to stack" << std::endl;

  // Define some variables
  int nfiles, num=0;
  int all_files=0;   // Becomes (1) if no more files remain
  long long nfilesl; // Function to_string cannot work with type "int"
  std::string file_in, suff, temp(in_pref);   
   // This loop scans the files 
   do {
    // Build the name of input files
    nfilesl=num+1;
    if(nfilesl >= 1 && nfilesl < 10) suff="_00"+std::to_string(nfilesl)+".H";
    else if(nfilesl >= 10 && nfilesl < 100) suff="_0"+std::to_string(nfilesl)+".H";
    else if(nfilesl >= 100 && nfilesl < 1000) suff="_"+std::to_string(nfilesl)+".H";
    else seperr("read_num_files: Cannot read thousands of input files!");
    file_in=temp+suff;
    num++;

    // Use this function to check if the file exists
    if(0 == auxin(file_in.c_str())) all_files=1;

    // If found, report the name of the file
    if(all_files == 0) {
      std::cout << "Found file: " << file_in << std::endl;
      auxclose(file_in.c_str());
    }
  } while(all_files != 1);
  if(num == 0) {
    seperr("read_num_files: No files found to stack! Check prefix.");
  }
  nfiles=(int)nfilesl-1;
  return nfiles;
}

void ParamMPI::read_history() {
  // Parameters from history files
  if(0 == hetch("n1","d",&n1))
    seperr("read_history: Parameter n1 not found\n");
  if(0 == hetch("n2","d",&n2))
    seperr("read_history: Parameter n2 not found\n");
  if(0 == hetch("n3","d",&n3))
    seperr("read_history: Parameter n3 not found\n");
  
  if(0 == hetch("o1","f",&o1))
    seperr("read_history: Parameter o1 not found\n");
  if(0 == hetch("o2","f",&o2))
    seperr("read_history: Parameter o2 not found\n");
  if(0 == hetch("o3","f",&o3))
    seperr("read_history: Parameter o3 not found\n");
  
  if(0 == hetch("d1","f",&d1))
    seperr("read_history: Parameter d1 not found\n");
  if(0 == hetch("d2","f",&d2))
    seperr("read_history: Parameter d2 not found\n");
  if(0 == hetch("d3","f",&d3))
    seperr("read_history: Parameter d3 not found\n");
  
  // Gather number of samples in offset and model coordinates
  n=n1*n2*n3;
  nsize=4*n;
}

void ParamMPI::allocate_vector(){
  // Allocate data vector
  allocV(d,n);
}

void ParamMPI::read_binary(const char* cur_file) {
  if(nsize != sreed(cur_file,d.data(),nsize)) {
    seperr("read_binary: Cannot read binary for file: %s",cur_file);
  }
}

void ParamMPI::write_history(std::string stack_file) {
  if(0 != auxputch("n1","d",&n1,stack_file.c_str()))
    seperr("write_history: Parameter n1 not found\n");
  if(0 != auxputch("n2","d",&n2,stack_file.c_str()))
    seperr("write_history: Parameter n2 not found\n");
  if(0 != auxputch("n3","d",&n3,stack_file.c_str()))
    seperr("write_history: Parameter n3 not found\n");
  
  if(0 != auxputch("o1","f",&o1,stack_file.c_str()))
    seperr("write_history: Parameter o1 not found\n");
  if(0 != auxputch("o2","f",&o2,stack_file.c_str()))
    seperr("write_history: Parameter o2 not found\n");
  if(0 != auxputch("o3","f",&o3,stack_file.c_str()))
    seperr("write_history: Parameter o3 not found\n");
  
  if(0 != auxputch("d1","f",&d1,stack_file.c_str()))
    seperr("write_history: Parameter d1 not found\n");
  if(0 != auxputch("d2","f",&d2,stack_file.c_str()))
    seperr("write_history: Parameter d2 not found\n");
  if(0 != auxputch("d3","f",&d3,stack_file.c_str()))
    seperr("write_history: Parameter d3 not found\n");
}

void ParamMPI::write_binary(std::vector<float> &d,std::string stack_file) {
  if(nsize != srite(stack_file.c_str(),d.data(),nsize)) {
    seperr("write_binary: Cannot write stack file");
  }
}
