#include "read.h"
#include "lap.h"
#include "prop_fwd.h"
#include "prop_bwd.h"
#include "alloc.h"
#include <iostream>
#include <string>
using namespace std;

// Initialize the backward propagation
void PropBwd::prop_bwd_init(Read par_in)
{
  par=par_in;
  pad =par.pad;
  zpad=par.zpad;
  nkgx=par.nkgx;
  nkz =par.nkz;
  nx  =par.nx;
  nz  =par.nz;
  nt  =par.nt;
  napL=par.napL;
  napR=par.napR;
  dt  =par.dt;
  dx  =par.dx;
  dz  =par.dz;
  first_cdp=par.first_cdp;
  last_cdp=par.last_cdp;

  // Some more "shorcuts"
  dt1=dt;
  dt2=dt1*dt1;
  dt3=dt2*dt1;
  dt4=dt2*dt2;

  // Allocate wavefield arrays
  alloc(dpxz,2,nkgx,nkz);
  alloc(prt,nkgx,nkz);
  alloc(prtp1,nkgx,nkz);
  alloc(prtp2,nkgx,nkz);
  alloc(prtX,nkgx,nkz);
  alloc(prtZ,nkgx,nkz);
  alloc(dprt,nkgx,nkz);
  alloc(dprtp1,nkgx,nkz);
  alloc(dprtp2,nkgx,nkz);
  alloc(d2prt,nkgx,nkz);
  alloc(d2prtp1,nkgx,nkz);
  alloc(d2prtp2,nkgx,nkz);

  // Allocate velocity arrays
  alloc(Vshot,nkgx,nkz);
  lap.lap_init(par); 
}

void PropBwd::init_rcv_wavefields() {
  // Initializa receiver wavefield arrays
  #pragma omp parallel for
  for(int jgx=0; jgx<par.nkgx; jgx++) {
    #pragma omp simd
    for(int jz=0; jz<par.nkz; jz++) {
      prt[jgx][jz]     = 0.0;
      prtp1[jgx][jz]   = 0.0;
      dprt[jgx][jz]    = 0.0;
      dprtp1[jgx][jz]  = 0.0;
      d2prt[jgx][jz]   = 0.0;
      d2prtp1[jgx][jz] = 0.0;

    }
  }

}

void PropBwd::bwd_prop(int adj) {
  // Update the wavefield frames
  #pragma omp parallel for
  for(int jgx=0; jgx<par.nkgx; jgx++) {
    #pragma omp simd
    for(int jz=0; jz<par.nkz; jz++) {
      prtp2[jgx][jz]   = prtp1[jgx][jz];
      prtp1[jgx][jz]   = prt[jgx][jz];
      dpxz[0][jgx][jz] = prtp1[jgx][jz];
      dprtp2[jgx][jz]  = dprtp1[jgx][jz];
      dprtp1[jgx][jz]  = dprt[jgx][jz];
      d2prtp2[jgx][jz] = d2prtp1[jgx][jz];
      d2prtp1[jgx][jz] = d2prt[jgx][jz];
    }
  }

  // Calculate the Laplacian
  lap.lap(dpxz,Vshot,adj);

  #pragma omp parallel for
  for(int jgx=0; jgx<par.nkgx; jgx++) {
    #pragma omp simd
    for(int jz=0; jz<par.nkz; jz++) {
      // Perform backward propagation
      prt[jgx][jz]=prtp1[jgx][jz]*2.0-prtp2[jgx][jz]
                  +dpxz[0][jgx][jz]*dt2+dpxz[1][jgx][jz]*dt4/12.0;

      // Obtain the 1st derivative of the backward propagated wavefield
//      dprt[jgx][jz]=dprtp2[jgx][jz]
//                   -2.0*dpxz[0][jgx][jz]*dt1-dpxz[1][jgx][jz]*dt3/3.0;

      // Obtain the 2nd derivative of the forward propagated wavefield
      d2prt[jgx][jz]=-d2prtp2[jgx][jz]
                 +2.0*dpxz[0][jgx][jz]+dpxz[1][jgx][jz]*dt2;
    }
  }
}


