#include "read.h"
#include "lap.h"
#include "prop_fwd.h"
#include "alloc.h"
#include <iostream>
#include <string>
using namespace std;

// Initialize the forward propagation
void PropFwd::prop_fwd_init(Read par_in) {
  // Shorcuts for frequently object's members invokation
  par=par_in;
  pad =par.pad;
  zpad=par.zpad;
  nkgx=par.nkgx;
  nkz =par.nkz;
  nx  =par.nx;
  nz  =par.nz;
  nt  =par.nt;
  napL=par.napL;
  napR=par.napR;
  dt  =par.dt;
  dx  =par.dx;
  dz  =par.dz;
  first_cdp=par.first_cdp;
  last_cdp=par.last_cdp;

  // Some more "shorcuts"
  dt1=dt;
  dt2=dt1*dt1;
  dt3=dt2*dt1;
  dt4=dt2*dt2;

  // Allocate wavefield arrays
  alloc(dpxz,2,nkgx,nkz);
  alloc(pft,nkgx,nkz);
  alloc(pftm1,nkgx,nkz);
  alloc(pftm2,nkgx,nkz);
  alloc(pftX,nkgx,nkz);
  alloc(pftZ,nkgx,nkz);
  alloc(dpft,nkgx,nkz);
  alloc(dpftm1,nkgx,nkz);
  alloc(dpftm2,nkgx,nkz);
  alloc(d2pft,nkgx,nkz);
  alloc(d2pftm1,nkgx,nkz);
  alloc(d2pftm2,nkgx,nkz);

  // Allocate velocity arrays
  alloc(Vshot,nkgx,nkz);

  // Initialize Laplacian
  lap.lap_init(par); 
}

void PropFwd::init_src_wavefields() {
  // Initialize source wavefield arrays
  #pragma omp parallel for
  for(int jgx=0; jgx<nkgx; jgx++) {
    #pragma omp simd
    for(int jz=0; jz<nkz; jz++) {
      pft[jgx][jz]     = 0.0;
      pftm1[jgx][jz]   = 0.0;
      dpft[jgx][jz]    = 0.0;
      dpftm1[jgx][jz]  = 0.0;
      d2pft[jgx][jz]   = 0.0;
      d2pftm1[jgx][jz] = 0.0;
    }
  }
}

void PropFwd::fwd_prop(int adj) {
  // Update the wavefield frames
  #pragma omp parallel for
  for(int jgx=0; jgx<nkgx; jgx++) {
    #pragma omp simd
    for(int jz=0; jz<nkz; jz++) {
      pftm2[jgx][jz]   = pftm1[jgx][jz];
      pftm1[jgx][jz]   = pft[jgx][jz];
      dpxz[0][jgx][jz] = pftm1[jgx][jz];
      dpftm2[jgx][jz]  = dpftm1[jgx][jz];
      dpftm1[jgx][jz]  = dpft[jgx][jz];
      d2pftm2[jgx][jz] = d2pftm1[jgx][jz];
      d2pftm1[jgx][jz] = d2pft[jgx][jz];
    }
  }

  // Calculate the Laplacian
  lap.lap(dpxz,Vshot,adj);

  #pragma omp parallel for
  for(int jgx=0; jgx<nkgx; jgx++) {
    #pragma omp simd
    for(int jz=0; jz<nkz; jz++) {
      // Perform forward propagation
      pft[jgx][jz]=pftm1[jgx][jz]*2.0-pftm2[jgx][jz]
                  +dpxz[0][jgx][jz]*dt2+dpxz[1][jgx][jz]*dt4/12.0;

      // Obtain the 1st derivative of the forward propagated wavefield
//      dpft[jgx][jz]=dpftm2[jgx][jz]
//                   +2.0*dpxz[0][jgx][jz]*dt1+dpxz[1][jgx][jz]*dt3/3.0;

      // Obtain the 2nd derivative of the forward propagated wavefield
      d2pft[jgx][jz]=-d2pftm2[jgx][jz]
                    +2.0*dpxz[0][jgx][jz]+dpxz[1][jgx][jz]*dt2;
    }
  }
}

