#include "read.h"
#include "lint1.h"
#include "grad.h"
#include "grad2.h"
#include "wave_prop.h"
#include "scale.h"
#include "scatter.h"
#include "taper.h"
#include <vector>
#include "alloc.h"
#include "allocv.h"
#include <cmath>
#include "snap.h"
#include "rtm.h"
#include "base_trismt.h"
#include "seplib.h"
#include "sep3d.h"
#include "sep_pars_external.h"
#include <iostream>
#include <numeric>
#include <algorithm>
#include "randomBound.h"

void Rtm::rtm_init(Read &par,WaveProp &fwd_in,WaveProp &bwd_in) {
  // Shorcuts for frequently object's members invokation
  pad=par.pad;
  zpad=par.zpad;
  nkgx=par.nkgx;
  nkz=par.nkz;
  nxE=par.nxE;
  nzE=par.nzE;
  nt=par.nt;
  napL=par.napL;
  napR=par.napR;
  dt=par.dt;
  dx=par.dx;
  dz=par.dz;
  use_taper=par.use_taper;
  first_cdp=par.first_cdp;
  last_cdp=par.last_cdp;
  nh=par.nh;
  dh=par.dh;
  hmax=par.hmax;
  datumS=par.datumS;
  datumR=par.datumR;
  shot_pos=par.shot_pos;
  eic=par.eic;
  ntSnap=0;
  fwd=&fwd_in; 
  bwd=&bwd_in; 
  dipole=par.dipole;

  // Allocate arrays for Xcorr imaging conditions
  allocV(num,nkgx*nkz*(2*nh+1));
//  allocV(den,nkgx*(2*nh+1)*nkz);
  allocV(ICorr,nxE*nzE*(2*nh+1));

  // Save the velocity frame in binary file
//  if(4*nkz*nkgx != srite("vrandom",(*fwd).Vshot.data(),4*nkz*nkgx))
//      seperr("rtm2d: Have trouble writing trace in random vel");

  // Vertical position of the source (dipole source)
  jsrc1=datumS  +pad+zpad+(pad+napL+shot_pos)*nkz;
  if(dipole) jsrc2=datumS+1+pad+zpad+(pad+napL+shot_pos)*nkz;

  // Reading data (traces from one shot gather)
  par.read_shot(0);

  /***** TEMPORAL: REMOVE AFTER TESTING  *****/
//  alloc(ptemp,nt+2,nkgx*nkz);
  /*******************************************/

}

void Rtm::rtm(Read &par) {
  // Initialize/update wavefield arrays
  (*fwd).init_wavefields();
  (*bwd).init_wavefields();

  std::vector<float> paux1(nkgx*nkz,0.);
  std::vector<float> paux2(nkgx*nkz,0.);
  std::vector<float> zero(nkgx*nkz,0.);

  // Initialize scalar
  scaleS.scale_init(par,(*fwd).Vel2,1);
  scaleR.scale_init(par,(*fwd).Vel2,1);

  // Initialize imaging conditions
  scatter.scatter_init(par,(*fwd).p);

  std::cout << "Perform reverse-time migration" << std::endl;

  for(int jt=0; jt<nt+2; jt++) {
    // Forward propagation of source wavefield
    if(jt>1) (*fwd).prop(0);

    if(jt>0 && jt<=nt) {
      // Prepare the dipole source
      paux1[jsrc1]= par.src[jt-1];
      if(dipole) paux1[jsrc2]=-par.src[jt-1];

      // Scale, then inject the source
      scaleS.scale(0,1,paux1,(*fwd).p);
    }

    // Save snapshots: source wavefield
    if(jt<nt) snapshot(par,jt,(*fwd).p,1,"snap1");

    /***** TEMPORAL: REMOVE AFTER TESTING  *****/
//    taper((*fwd).pm2,(*fwd).pm1,(*fwd).p);
//    for(int jj=0; jj<(*fwd).p.size(); jj++) {
//      ptemp[jt][jj]=(*fwd).p[jj];
//    }
//   int n=nkgx*nkz;
//   if(4*n!=srite("src_wfd",(*fwd).p.data(),4*n))
//     seperr("rtm: Cannot write source wavefield in file");
   /*******************************************/
  }

  // Swap source wavefield time frames to propagate backwards 
  (*fwd).p.swap((*fwd).pm1);

  paux1=zero;
  paux2=zero;

  for(int jt=nt+1; jt>=0; jt--) {
    // Backward propagation of receiver wavefield
    if(jt<nt-0) (*bwd).prop(1);

    if(jt>0 && jt<nt+1 && remainder(jt,par.scatStep)==0) {
      // Prepare the dipole data, and inject
      #pragma omp parallel for
      for(int jgx=0; jgx<nkgx; jgx++) {
        int j1=datumR  +pad+zpad+jgx*nkz;
        int j2=datumR+1+pad+zpad+jgx*nkz;
        int j3=jt-1+jgx*nt;
        (*bwd).p[j1]+=par.rcv[j3];
        if(dipole) (*bwd).p[j2]-=par.rcv[j3];
      }
    }

    // Scale the rec. wavefield after propagation
    if(remainder(jt,par.scatStep)==0) scaleR.scale(1,0,paux2,(*bwd).p);

    // Backward propagation of source wavefield
    (*fwd).prop(0);
    /**** TEMPORAL: REMOVE AFTER TESTING ****/
//    for(int jj=0; jj<(*fwd).p.size(); jj++) {
//      (*fwd).p[jj]=ptemp[jt][jj];
//    }
//   int n=nkgx*nkz;
//   if(-1==sseek_block("src_wfd",jt,4*n,0)) 
//     seperr("rtm: Cannot position within source wavefield in file");
//   if(4*n!=sreed("src_wfd",(*fwd).p.data(),4*n))
//     seperr("rtm: Cannot reed source wavefield in file");
    /****************************************/

    if(jt>0 && jt<nt+1) {
      // Prepare the dipole source wavelet
      paux1[jsrc1]= par.src[jt-1];
      if(dipole) paux1[jsrc2]=-par.src[jt-1];

      // Scale, then inject the source
      scaleS.scale(0,1,paux1,(*fwd).p);
    }

    // Save snapshots: source wavefield backwards
    snapshot(par,jt,(*fwd).p,0,"snap2");

    // Use damping function in receiver propagation wavefields
    if(use_taper==1) {
      taper((*bwd).pm2,(*bwd).pm1,(*bwd).p);
    }

    // Save snapshots: receiver wavefield
    snapshot(par,jt,paux2,0,"snap3");

    if(jt>0 && jt<nt && remainder(jt,par.scatStep)==0) {
      // Apply imaging condition after scaling receiver wavefield (ext. domain)
      scatter.scatter(1,1,num,paux2); 
    }
  }
}

void Rtm::build_images(Read par) {
  for(int jh=0; jh<2*nh+1; jh++){
//  #pragma omp parallel for
    for(int jgx=0; jgx<nkgx; jgx++){
      int jx=floor(par.sgx[jgx]/dx)+pad+napL;
      if(jx>=0 && jx<nxE) {
//        #pragma omp simd
        for(int jz=0; jz<nkz; jz++) {
          int j1=jz+jgx*nkz+jh*nkgx*nkz;
          int j2=jz+jx*nzE+jh*nxE*nzE;
          ICorr[j2]=num[j1];
        }
      }
    }
  }
}
