#include "read.h"
#include "scale.h"
#include <vector>
#include "allocv.h"
#include <iostream>

void Scale::scale_init(Read &par,std::vector<float> &Vel2,int Nyq) {
  // Some shortcuts
  nkgx=par.nkgx;
  nkz=par.nkz;

  // Allocate scaling factor
  allocV(scl,Vel2.size());

  int scatStep=1;
  if(Nyq == 1) scatStep=par.scatStep;

  // Prepare scaling factors
  #pragma omp parallel for
  for(int jj=0; jj<Vel2.size(); jj++) {
    scl[jj]=-Vel2[jj]*par.dt*par.dt*scatStep;
  }
}
  
void Scale::scale(int adj,int add,
                  std::vector<float> &m,
                  std::vector<float> &d) {
  if(adj==1) {
    if(add==0) {
      #pragma omp parallel for
      for(int jj=0; jj<m.size(); jj++) {
        m[jj]=0.;
      }
    } 
    #pragma omp parallel for schedule(static)
    for(int jgx=0; jgx<nkgx; jgx++) {
      #pragma omp simd simdlen(8)
      for(int jz=0; jz<nkz; jz++) {
        int jj=jz+jgx*nkz;
        m[jj]+=d[jj]*scl[jj];
      }
    }
  }
  else {
    if(add==0) {
      #pragma omp parallel for
      for(int jj=0; jj<d.size(); jj++) {
        d[jj]=0.;
      }
    } 
    #pragma omp parallel for schedule(static)
    for(int jgx=0; jgx<nkgx; jgx++) {
      #pragma omp simd simdlen(8)
      for(int jz=0; jz<nkz; jz++) {
        int jj=jz+jgx*nkz;
        d[jj]+=m[jj]*scl[jj];
      }
    }
  }
}
