#include "read.h"
#include "wave_prop.h"
#include "scatter.h"
#include <vector>
#include "allocv.h"

void Scatter::scatter_init(Read &par,
                           std::vector<float> &psrc_in) {
  nkgx=par.nkgx;
  nkz=par.nkz;
  nh=par.nh;
  dh=par.dh;
  hmax=par.hmax;
  psrc=&psrc_in;
}

void Scatter::scatter(int adj,int add,
                      std::vector<float> &m,
                      std::vector<float> &d) {
  if(adj==1) { // Imaging condition
    if(add==0) {
      #pragma omp parallel for
      for(int jj=0; jj<m.size(); jj++) {
        m[jj]=0.;
      }
    } 
//    #pragma omp parallel for schedule(guided) collapse(2)
    for(int jh=-nh; jh<=nh; jh++) {
    #pragma omp parallel for schedule(static) 
      for(int jgx=0+hmax; jgx<nkgx-hmax; jgx++) {
        #pragma omp simd simdlen(8)
        for(int jz=0; jz<nkz; jz++) {
          int j1=jz+jgx*nkz+(jh+nh)*nkgx*nkz;
          int j2=jz+(jgx-(int)(jh*dh))*nkz;
          int j3=jz+(jgx+(int)(jh*dh))*nkz;
          m[j1]+=(*psrc)[j2]*d[j3]; 
        }
      }
    }
  }
  else {
    if(add==0) { // Scattering
      #pragma omp parallel for
      for(int jj=0; jj<d.size(); jj++) {
        d[jj]=0.;
      }
    } 
//    #pragma omp parallel for schedule(guided) collapse(2)
    for(int jh=-nh; jh<=nh; jh++) {
      #pragma omp parallel for schedule(static)
      for(int jgx=0+hmax; jgx<nkgx-hmax; jgx++) {
        #pragma omp simd simdlen(8)
        for(int jz=0; jz<nkz; jz++) {
          int j1=jz+jgx*nkz+(jh+nh)*nkgx*nkz;
          int j2=jz+(jgx-(int)(jh*dh))*nkz;
          int j3=jz+(jgx+(int)(jh*dh))*nkz;
          d[j3]+=(*psrc)[j2]*m[j1]; 
        }
      }
    }
  }
}
