#include "read.h"
#include "taper.h"
#include <iostream>
#include <vector>
#include <string>
#include <cmath>
using namespace std;

Read par;
float *zxtaper, **ctaper;

void taper_init(Read &par_in) {
  if(par_in.use_taper==1) {
    par=par_in;
    int pad=par.pad;
    zxtaper=new float[pad]; 
    ctaper=new float*[pad]; 
    for(int jj=0; jj<pad; jj++)
      ctaper[jj]=new float[pad];
  
    for(int jj=0; jj<pad; jj++)
      zxtaper[jj] = 0.99+0.01*cos(PI*(pad-jj-1)/pad);
    for(int jx=0; jx<pad; jx++) {
      for(int jz=0; jz<pad; jz++) {
        float dist=sqrt(1.001*(pad-jx-1)*(pad-jx-1)/pad/pad+
                     0.999999*(pad-jz-1)*(pad-jz-1)/pad/pad);
        ctaper[jx][jz]=0.99+0.01*cos(PI*dist);
      }
    }
  }
}

//void taper(vector<vector<float> >&prev,vector<vector<float> >&cur,vector<vector<float> >&next) {
void taper(vector<float>&prev,vector<float>&cur,vector<float>&next) {
  int pad=par.pad;
  int nkgx=par.nkgx;
  int nkz =par.nkz;
  #pragma omp parallel for
  for(int jx=0; jx<pad; jx++) {
    #pragma omp simd
    for(int jz=0; jz<pad; jz++) {
      int j1=jz+jx*nkz;
      int j2=jz+(nkgx-1-jx)*nkz;
      int j3=(nkz-1-jz)+(nkgx-1-jx)*nkz;
      int j4=(nkz-1-jz)+jx*nkz;
      prev[j1] *= ctaper[jx][jx];
      prev[j2] *= ctaper[jx][jx];
      prev[j3] *= ctaper[jx][jx];
      prev[j4] *= ctaper[jx][jx];
//      prev[jx][jz]          = prev[jx][jz]*ctaper[jx][jx];
//      prev[nkgx-1-jx][jz]     *= ctaper[jx][jx];
//      prev[nkgx-1-jx][nkz-1-jz] *= ctaper[jx][jx];
//      prev[jx][nkz-1-jz]      *= ctaper[jx][jx];

      cur[j1] *= ctaper[jx][jx];
      cur[j2] *= ctaper[jx][jx];
      cur[j3] *= ctaper[jx][jx];
      cur[j4] *= ctaper[jx][jx];
//      cur[jx][jz]          *=ctaper[jx][jx];
//      cur[nkgx-1-jx][jz]     *=ctaper[jx][jx];
//      cur[nkgx-1-jx][nkz-1-jz] *=ctaper[jx][jx];
//      cur[jx][nkz-1-jz]      *=ctaper[jx][jx];

      next[j1] *= ctaper[jx][jx];
      next[j2] *= ctaper[jx][jx];
      next[j3] *= ctaper[jx][jx];
      next[j4] *= ctaper[jx][jx];
//      next[jx][jz]          *= ctaper[jx][jx];
//      next[nkgx-1-jx][jz]     *= ctaper[jx][jx];
//      next[nkgx-1-jx][nkz-1-jz] *= ctaper[jx][jx];
//      next[jx][nkz-1-jz]      *= ctaper[jx][jx];
    }
  }
  #pragma omp parallel for
  for(int jx=0; jx<pad; jx++) {
    #pragma omp simd
    for(int jz=pad; jz<nkz-pad; jz++) {
      int j1=jz+jx*nkz;
      int j2=jz+(jx+nkgx-pad)*nkz;
      prev[j1] *= zxtaper[jx];
      prev[j2] *= zxtaper[pad-jx-1];
//      prev[jx][jz]          *=zxtaper[jx];
//      prev[jx+nkgx-pad][jz] *=zxtaper[pad-jx-1];

      cur[j1] *= zxtaper[jx];
      cur[j2] *= zxtaper[pad-jx-1];
//      cur[jx][jz]           *=zxtaper[jx];
//      cur[jx+nkgx-pad][jz]  *=zxtaper[pad-jx-1];

      next[j1] *= zxtaper[jx];
      next[j2] *= zxtaper[pad-jx-1];
//      next[jx][jz]          *=zxtaper[jx];
//      next[jx+nkgx-pad][jz] *=zxtaper[pad-jx-1];
    }
  }
  #pragma omp parallel for
  for(int jx=pad; jx<nkgx-pad; jx++) {
    #pragma omp simd
    for(int jz=0; jz<pad; jz++) {
      int j1=jz+jx*nkz;
      int j2=jz+nkz-pad+jx*nkz;
      prev[j1] *= zxtaper[jz];
      prev[j2] *= zxtaper[pad-jz-1];
//      prev[jx][jz]         *=zxtaper[jz];
//      prev[jx][jz+nkz-pad] *=zxtaper[pad-jz-1];

      cur[j1] *= zxtaper[jz];
      cur[j2] *= zxtaper[pad-jz-1];
//      cur[jx][jz]          *=zxtaper[jz];
//      cur[jx][jz+nkz-pad]  *=zxtaper[pad-jz-1];

      next[j1] *= zxtaper[jz];
      next[j2] *= zxtaper[pad-jz-1];
//      next[jx][jz]         *=zxtaper[jz];
//      next[jx][jz+nkz-pad] *=zxtaper[pad-jz-1];
    }
  }
}

