#include "read.h"
#include "grad.h"
#include "wave_prop.h"
#include "scatter.h"
#include "scale.h"
#include "wemva.h"
#include "operator.h"
#include "dso.h"
#include "wemva_op.h"
#include <vector>
#include <iostream>

Wemva_op::Wemva_op(Read &par_in,Wemva &wemva_in,
                   Wemva &wemva_adj_in,
                   WaveProp &fwd,WaveProp &scat,WaveProp &bwd,
                   Dso &dso_in,int doit) {
  if(doit==1) {
    par=&par_in;
    wemva=&wemva_in;
    wemva_adj=&wemva_adj_in;
    dso=&dso_in;
    nkz=(*par).nkz;
    nkgx=(*par).nkgx;
    nh=(*par).nh;
    (*wemva).wemva_init(*par,fwd,bwd,scat);
    (*wemva_adj).wemva_init(*par,fwd,bwd,scat);
  }
}

Wemva_op::~Wemva_op() {}

void Wemva_op::op(int adj,int add,
                  std::vector<float> &model,
                  std::vector<float> &data) {
  if(adj == 1) {
    std::vector<float> temp(data.size(),0.);
    if(add == 0) {
      #pragma omp parallel for
      for(int jj=0; jj<model.size(); jj++) {
        model[jj]=0.;
      }
    }

    // Apply DSO (flips the sign only if nh=0)
    (*dso).op(1,0,(*wemva_adj).pshot,data);
//    (*dso).op(1,0,temp,data);

    // Apply differential in subsurface offset
//    (*diff).op(1,0,(*wemva_adj).pshot,temp);

    // Run adjoint WEMVA
    (*wemva_adj).wemva(*par);

//    #pragma omp parallel for
    for(int jgx=0; jgx<nkgx; jgx++) {
//      #pragma omp simd
      for(int jz=0; jz<nkz; jz++) {
        int jj=jz+jgx*nkz+nh*nkgx*nkz;
        model[jj]+=(*wemva_adj).pert[jj] ;
      }
    }
  }
  else {
    std::vector<float> temp(data.size(),0.);
    if(add == 0) {
      #pragma omp parallel for
      for(int jj=0; jj<data.size(); jj++) {
        data[jj] = 0.;
      }
    }
    #pragma omp parallel for
    for(int jj=0; jj<(*wemva).pshot.size(); jj++) {
      (*wemva).pshot[jj]=0;
    }

//    #pragma omp parallel for
    for(int jgx=0; jgx<nkgx; jgx++) {
//      #pragma omp simd
      for(int jz=0; jz<nkz; jz++) {
        int jj=jz+jgx*nkz+nh*nkgx*nkz;
        (*wemva).pshot[jj]=model[jj];
      }
    }

    // Run forward WEMVA
    (*wemva).wemva(*par);

    // Apply differential in subsurface offset
//    (*diff).op(0,0,(*wemva).pert,temp);

    // Apply DSO (flips the sign only if nh=0)
    (*dso).op(0,1,(*wemva).pert,data);
//    (*dso).op(0,1,(*wemva).pert,data);
//    for(int jj=0; jj< data.size(); jj++) {
//      data[jj]+=(*wemva).pert[jj];
//    }
  }
}

