#include "interp.h"
#include "read.h"
#include "alloc.h"
#include "seplib.h"
#include "sep3d.h"
#include "sep_pars_external.h"
#include "write.h"
#include <iostream>
#include <string.h>
#include <assert.h>

void Write::write_init(Read par) {
  // Define some "shorcuts "
  pad=par.pad;
  zpad=par.zpad;
  nkgx=par.nkgx;
  nkz=par.nkz;
  ngx=par.ngx;
  nx=par.nx;
  nz=par.nz;
  nxE=par.nxE;
  nzE=par.nzE;
  ntData=par.ntData;
  nt=par.nt;
  ntSnap=par.ntSnap;
  napL=par.napL;
  napR=par.napR;
  dtData=par.dtData;
  dt=par.dt;
  dx=par.dx;
  dz=par.dz;
  o1=par.o1;
  o2=par.o2;
  first_cdp=par.first_cdp;
  nh=2*par.nh+1;
  dh=par.dx*par.dh;
  oh=-par.hmax*par.dx;
  jtSnap=par.jtSnap;
  tIntFact=par.tIntFact;
}

void Write::write_image(Read par,float *I ,char *out_fn) {
  // Write snapshot and velocity history files parameters
  write_snap("snap1");
  write_snap("snap2");
  write_snap("snap3");
  write_vshot();
  // Write history files
  std::cout << "write_image: Write the history files" << std::endl;  
  if(0 != auxputch("n1","d",&nzE,out_fn))
    seperr("write_image: Cannot write n1 upon history file"); 
  if(0 != auxputch("n2","d",&nxE,out_fn))
    seperr("write_image: Cannot write n2 upon history file"); 
  if(0 != auxputch("n3","d",&nh,out_fn))
    seperr("write_image: Cannot write n3 upon history file"); 
  if(0 != auxputch("d1","f",&dz,out_fn))
    seperr("write_image: Cannot write d1 upon history file"); 
  if(0 != auxputch("d2","f",&dx,out_fn))
    seperr("write_image: Cannot write d2 upon history file"); 
  if(0 != auxputch("d3","f",&dh,out_fn))
    seperr("write_image: Cannot write d3 upon history file"); 
  if(0 != auxputch("dz","f",&dz,out_fn))
    seperr("write_image: Cannot write d1 upon history file"); 
  if(0 != auxputch("dcdp","f",&dx,out_fn))
    seperr("write_image: Cannot write dcdp upon history file"); 
  float o1=dz*(-zpad-pad);
  if(0 != auxputch("o1","f",&o1,out_fn))
    seperr("write_image: Cannot write o1 upon history file"); 
  float o2=dx*(-pad-napL);
  if(0 != auxputch("o2","f",&o2,out_fn))
    seperr("write_image: Cannot write o2 upon history file"); 
  if(0 != auxputch("o3","f",&oh,out_fn))
    seperr("write_image: Cannot write o3 upon history file"); 
  char_aux="depth";
  if(0 != auxputch("label1","s",char_aux,out_fn))
    seperr("write_image: Cannot write label1 upon history file"); 
  char_aux="distance";
  if(0 != auxputch("label2","s",char_aux,out_fn))
    seperr("write_image: Cannot write label2 upon history file"); 
  char_aux="offset";
  if(0 != auxputch("label3","s",char_aux,out_fn))
    seperr("write_image: Cannot write label3 upon history file"); 

  // Initialize amplitudes and non-zero samples
  amp_min= 99999.0;
  amp_max=-99999.0;
  non_zero=0;

  // Write the output onto the binary
  int n=nxE*nzE*(2*par.nh+1);
  if(4*n != srite(out_fn,I,4*n))
    seperr("write_image: Have trouble writing trace");
  for(int jj=0; jj<n; jj++) {
    // Test for NaNs
    assert(!isnan(I[jj]) && 
          "write_image: There are NaNs in the output");
    
    // Obtain amplitudes and non-zero samples
    if(amp_min > I[jj]) amp_min = I[jj];
    if(amp_max < I[jj]) amp_max = I[jj];
    if(I[jj] != 0.0) {
      amp_rms+= I[jj]*I[jj];
      amp_mean+=I[jj];
      non_zero++;
    }
  }

  // Test for "only zeros" outcome
//  assert(non_zero != 0 && "write_image: The output is made of zeros!");

  // Write amplitude information onto history file
  write_amp(out_fn);
}

void Write::write_pert(Read par,float *P ,char *out_fn) {
  // Write snapshot and velocity history files parameters
  write_snap("snap1");
  write_snap("snap2");
  write_snap("snap3");
//  write_vshot();
  // Write history files
  std::cout << "write_pert: Write the history files" << std::endl; 
  if(0 != auxputch("n1","d",&nzE,out_fn))
    seperr("write_pert: Cannot write n1 upon history file"); 
  if(0 != auxputch("n2","d",&nxE,out_fn))
    seperr("write_pert: Cannot write n3 upon history file"); 
  int n3=1;
  if(0 != auxputch("n3","d",&n3,out_fn))
    seperr("write_pert: Cannot write n3 upon history file"); 
  if(0 != auxputch("d1","f",&dz,out_fn))
    seperr("write_pert: Cannot write d1 upon history file"); 
  if(0 != auxputch("d2","f",&dx,out_fn))
    seperr("write_pert: Cannot write d3 upon history file"); 
  int d3=1;
  if(0 != auxputch("d3","f",&d3,out_fn))
    seperr("write_pert: Cannot write d3 upon history file"); 
  if(0 != auxputch("dz","f",&dz,out_fn))
    seperr("write_pert: Cannot write d1 upon history file"); 
  if(0 != auxputch("dcdp","f",&dx,out_fn))
    seperr("write_pert: Cannot write dcdp upon history file"); 
  float o1=dz*(-zpad-pad);
  if(0 != auxputch("o1","f",&o1,out_fn))
    seperr("write_pert: Cannot write o2 upon history file"); 
  float o2=dx*(-pad-napL);
  if(0 != auxputch("o2","f",&o2,out_fn))
    seperr("write_pert: Cannot write o3 upon history file"); 
  int o3=1;
  if(0 != auxputch("o3","f",&o3,out_fn))
    seperr("write_pert: Cannot write o3 upon history file"); 
  char_aux="depth";
  if(0 != auxputch("label1","s",char_aux,out_fn))
    seperr("write_pert: Cannot write label1 upon history file"); 
  char_aux="distance";
  if(0 != auxputch("label2","s",char_aux,out_fn))
    seperr("write_pert: Cannot write label3 upon history file"); 

  // Initialize amplitudes and non-zero samples
  amp_min= 99999.0;
  amp_max=-99999.0;
  non_zero=0;

  // Write the output onto the binary
  float *Q = new float[nxE*nzE];
  for(int jx=0; jx<nxE; jx++) {
    for(int jz=0; jz<nzE; jz++) {
      int j1=jz+jx*nzE+par.nh*nxE*nzE;
      int j2=jz+jx*nzE;
      Q[j2]=P[j1];
    }
  }
  int n=nxE*nzE;
  if(4*n != srite(out_fn,Q,4*n))
    seperr("write_history: Have trouble writing trace");
  for(int jj=0; jj<n; jj++) {
    // Test for NaNs
    assert(!isnan(Q[jj]) && 
      "write_history: There are NaNs in the output");
    
    // Obtain amplitudes and non-zero samples
    if(amp_min > Q[jj]) amp_min = Q[jj];
    if(amp_max < Q[jj]) amp_max = Q[jj];
    if(Q[jj] != 0.0) {
      amp_rms+= Q[jj]*Q[jj];
      amp_mean+=Q[jj];
      non_zero++;
    }
  }
  delete Q;

  // Test for "only zeros" outcome
  assert(non_zero != 0 && "write_history: The output is made of zeros!");

  // Write amplitude information onto history file
  write_amp(out_fn);

}

void Write::write_psf(Read par,float *P) {
  // Write history files
  write_image(par,P,"psf");

  int n1=par.npsfz;
  if(0 != auxputch("nzc","d",&n1,"psf"))
    seperr("write_psf: Cannot write nzc upon history file"); 
  int n2=par.npsfx;
  if(0 != auxputch("nxc","d",&n2,"psf"))
    seperr("write_psf: Cannot write nxc upon history file"); 
  std::cout << "write_psf: Write the history files" << std::endl;  
}

void Write::write_data(Read par,std::vector<float> D ,char *out_fn) {
  // Write snapshot and velocity history files parameters
  write_snap("snap1");
  write_snap("snap2");
  write_snap("snap3");
//  write_vshot();

  // Write history files
  std::cout << "write_data: Write the history files" << std::endl;  
  if(0 != auxputch("n1","d",&ntData,out_fn))
    seperr("write_data: Cannot write n1 upon history file"); 
  if(0 != auxputch("d1","f",&dtData,out_fn))
    seperr("write_data: Cannot write d1 upon history file"); 
  float o1=0;
  if(0 != auxputch("o1","f",&o1,out_fn))
    seperr("write_data: Cannot write o1 upon history file"); 
  char_aux="time";
  if(0 != auxputch("label1","s",char_aux,out_fn))
    seperr("write_data: Cannot write label1 upon history file"); 

  int n2=nkgx;
  if(0 != auxputch("n2","d",&n2,out_fn))
    seperr("write_data: Cannot write n1 upon history file"); 
  if(0 != auxputch("d2","f",&onef,out_fn))
    seperr("write_data: Cannot write d1 upon history file"); 
  float o2=1;
  if(0 != auxputch("o2","f",&o2,out_fn))
    seperr("write_data: Cannot write o1 upon history file"); 
  char_aux="trace_no";
  if(0 != auxputch("label2","s",char_aux,out_fn))
    seperr("write_data: Cannot write label1 upon history file"); 


  if(0 != auxputch("n3","d",&one,out_fn))
    seperr("write_data: Cannot write n1 upon history file"); 
  if(0 != auxputch("d3","f",&onef,out_fn))
    seperr("write_data: Cannot write d1 upon history file"); 
  float o3=0;
  if(0 != auxputch("o3","f",&o3,out_fn))
    seperr("write_data: Cannot write o1 upon history file"); 
  char_aux="none";
  if(0 != auxputch("label3","s",char_aux,out_fn))
    seperr("write_data: Cannot write label1 upon history file"); 

  // Initialize amplitudes and non-zero samples
  amp_min= 99999.0;
  amp_max=-99999.0;
  non_zero=0;

  // Write the output onto the binary
  std::vector<float> Data(ntData);
  int doit=0;
  if(tIntFact > 1) doit=1;
  Interp interp(tIntFact,doit);
  if(par.adj == 0) {
    for(int jgx=0; jgx<nkgx; jgx++) {
      std::vector<float> Dvec(nt);
      for(int jt=0; jt<nt; jt++) {
        int jj=jt+jgx*nt;
        Dvec[jt]=D[jj];
      }
      if(tIntFact > 1) {
        interp.op(1,0,Data,Dvec);
      }
      else {
        Data.swap(Dvec);
      }
      for(int jt=0; jt<ntData; jt++) {
        // Test for NaNs
        assert(!isnan(Data[jt]) && 
          "write_data: There are NaNs in the output");
        
        // Obtain amplitudes and non-zero samples
        if(amp_min > Data[jt]) amp_min = Data[jt];
        if(amp_max < Data[jt]) amp_max = Data[jt];
        if(Data[jt] != 0.0) {
          amp_rms+= Data[jt]*Data[jt];
          amp_mean+=Data[jt];
          non_zero++;
        }
      }
      if(4*ntData != srite(out_fn,Data.data(),4*ntData))
        seperr("write_data: Have trouble writing trace");
    }
  }
  else {
    std::vector<float> Dvec(nt);
    for(int jt=0; jt<nt; jt++) {
      Dvec[jt]=D[jt];
    }
    if(tIntFact > 1) {
      interp.op(1,0,Data,Dvec);
    }
    else {
      for(int jt=0; jt<nt; jt++) {
        Data[jt]=D[jt];
      }
    }
    for(int jt=0; jt<ntData; jt++) {
      // Test for NaNs
      assert(!isnan(Data[jt]) && 
        "write_data: There are NaNs in the output");
      
      // Obtain amplitudes and non-zero samples
      if(amp_min > Data[jt]) amp_min = Data[jt];
      if(amp_max < Data[jt]) amp_max = Data[jt];
      if(Data[jt] != 0.0) {
        amp_rms+= Data[jt]*Data[jt];
        amp_mean+=Data[jt];
        non_zero++;
      }
    }
    if(4*ntData != srite(out_fn,Data.data(),4*ntData))
      seperr("write_data: Have trouble writing trace");
  }

  // Test for "only zeros" outcome
  assert(non_zero != 0 && "write_data: The output is made of zeros!");

  // Write amplitude information onto history file
  write_amp(out_fn);

  // Obtain rms and mean amplitudes
  amp_rms =sqrt(amp_rms/non_zero);
  amp_mean=amp_mean/non_zero;

}

void Write::write_data_headers(Read par, char* out_fn) {
   // Write the headers' keys for synthetic data
   std::cout << "write_data_headers: Write headers on modeled data" << std::endl;
   key=5; 
   if(0 != sep_put_number_keys(out_fn,&key))
     seperr("write_data_headers: Cannot write number of keys on the hff (.H@@) file!");
   key=1;
   if(0 != sep_put_key(out_fn,"sx","scalar_float","xdr_float",&key)) 
     seperr("write_data_headers: sx key index cannot be written in modeled data file!");
   key=2;
   if(0 != sep_put_key(out_fn,"gx","scalar_float","xdr_float",&key))
      seperr("write_data_headers: gx key index cannot be written in modeled data file!");
   key=3;
   if(0 != sep_put_key(out_fn,"sy","scalar_float","xdr_float",&key))
     seperr("write_data_headers: sy key index cannot be written in modeled data file!");
   key=4;
   if(0 != sep_put_key(out_fn,"gy","scalar_float","xdr_float",&key))
     seperr("write_data_headers: gy key index cannot be written in modeled data file!");
   key=5;
   if(0 != sep_put_key(out_fn,"scalco","scalar_float","xdr_float",&key))
     seperr("write_data_headers: scalco key cannot be writen in modeled data file!");
   scalcof = par.scalco;
   if(par.scalco < 0)
     scalcof = -1.0/scalcof;

   // Allocate the header array
   alloc(header_out,nkgx,5);
   
   // Populate the header array
   for(int jgx=0; jgx<nkgx; jgx++) {
     header_out[jgx][0] = par.sx[jgx]/scalcof;
     header_out[jgx][1] = par.gx[jgx]/scalcof;
     header_out[jgx][2] = 0;
     header_out[jgx][3] = 0;
     header_out[jgx][4] = par.scalco;
   }

   // Write the header values onto the header file
  if(0 != sep_put_data_axis_par(out_fn,&two,&nkgx,&zero,&onef,"trace_number"))
    seperr("write_data_headers: Cannot write data_axis_par!");
  if(0 != sep_put_header_axis_par(out_fn,&two,&nkgx,&zero,&onef,"trace_number"))
    seperr("write_data_headers: Cannot write header_axis_par!");
  int one=1;
  for(int jgx=0; jgx<nkgx; jgx++) {
    if(0 != sep_put_val_headers(out_fn,&jgx,&one,header_out[jgx]))
      seperr("write_data_headers: Cannot write to output data headers!");
  }

  delete header_out;  
}

void Write::write_error_res(Read par,int niter,float *error,char *error_fn) {
  if(0 != auxputch("n1","d",&niter,error_fn))
    seperr("write_error_res: Cannot write n1 upon history file"); 
  int uno=1;
  if(0 != auxputch("d1","d",&uno,error_fn))
    seperr("write_error_res: Cannot write d1 upon history file"); 
  if(0 != auxputch("o1","d",&uno,error_fn))
    seperr("write_error_res: Cannot write n1 upon history file"); 

  if(4*niter != srite(error_fn,error,4*niter))
    seperr("write_error_res: Have trouble writing error_res");
}

void Write::write_hess_image(Read par,float *I,int adj) {
  if(adj==1) {
  // Re-use conventional, write image
    write_image(par,I,"hessian");
  }
  else {
    char *out_fn="hessian";
    // Write history files
    std::cout << "write_hess_image: Write the history files" << std::endl;  
    int n1=nzE+2*par.nlagz;
    if(0 != auxputch("n1","d",&n1,out_fn))
      seperr("write_hess_image: Cannot write n1 upon history file"); 
    int n2=nxE+2*par.nlagx;
    if(0 != auxputch("n2","d",&n2,out_fn))
      seperr("write_hess_image: Cannot write n2 upon history file"); 
    if(0 != auxputch("n3","d",&nh,out_fn))
      seperr("write_hess_image: Cannot write n3 upon history file"); 
    if(0 != auxputch("d1","f",&dz,out_fn))
      seperr("write_hess_image: Cannot write d1 upon history file"); 
    if(0 != auxputch("d2","f",&dx,out_fn))
      seperr("write_hess_image: Cannot write d2 upon history file"); 
    if(0 != auxputch("d3","f",&dh,out_fn))
      seperr("write_hess_image: Cannot write d3 upon history file"); 
    float o1=dz*(-zpad-pad-par.nlagz);
    if(0 != auxputch("o1","f",&o1,out_fn))
      seperr("write_hess_image: Cannot write o1 upon history file"); 
    float o2=dx*(-pad-napL-par.nlagx);
    if(0 != auxputch("o2","f",&o2,out_fn))
      seperr("write_hess_image: Cannot write o2 upon history file"); 
    if(0 != auxputch("o3","f",&oh,out_fn))
      seperr("write_hess_image: Cannot write o3 upon history file"); 
    char_aux="depth";
    if(0 != auxputch("label1","s",char_aux,out_fn))
      seperr("write_hess_image: Cannot write label1 upon history file"); 
    char_aux="distance";
    if(0 != auxputch("label2","s",char_aux,out_fn))
      seperr("write_hess_image: Cannot write label2 upon history file"); 
    char_aux="offset";
    if(0 != auxputch("label3","s",char_aux,out_fn))
      seperr("write_hess_image: Cannot write label3 upon history file"); 
  
    // Initialize amplitudes and non-zero samples
    amp_min= 99999.0;
    amp_max=-99999.0;
    non_zero=0;
  
    // Write the output onto the binary
    int n=(nxE+2*par.nlagx)*(nzE+2*par.nlagz)*(2*par.nh+1);
    if(4*n != srite(out_fn,I,4*n))
      seperr("write_hess_image: Have trouble writing trace");
    for(int jj=0; jj<n; jj++) {
      // Test for NaNs
      assert(!isnan(I[jj]) && 
            "write_hess_image: There are NaNs in the output");
      
      // Obtain amplitudes and non-zero samples
      if(amp_min > I[jj]) amp_min = I[jj];
      if(amp_max < I[jj]) amp_max = I[jj];
      if(I[jj] != 0.0) {
        amp_rms+= I[jj]*I[jj];
        amp_mean+=I[jj];
        non_zero++;
      }
    }
  
    // Test for "only zeros" outcome
  //  assert(non_zero != 0 && "write_hess_image: The output is made of zeros!");
  
    // Write amplitude information onto history file
    write_amp(out_fn);
      
  }
}

void Write::write_snap(char *snap) {
  // Write the snapshots history files
  if(0 != auxputch("n1","d",&nkz,snap))
    seperr("write_snap: Cannot write n1 upon history file"); 
  int n=nxE;
  if(0 != auxputch("n2","d",&n,snap))
    seperr("write_snap: Cannot write n2 upon history file"); 
  if(0 != auxputch("n3","d",&ntSnap,snap))
    seperr("write_snap: Cannot write n3 upon history file"); 
  if(0 != auxputch("d1","f",&dz,snap))
    seperr("write_snap: Cannot write d1 upon history file"); 
  if(0 != auxputch("d2","f",&dx,snap))
    seperr("write_snap: Cannot write d2 upon history file"); 
  float dtSnap=dt*jtSnap;
  if(0 != auxputch("d3","f",&dtSnap,snap))
    seperr("write_snap: Cannot write d3 upon history file"); 
  float o1=dz*(-zpad-pad);
  if(0 != auxputch("o1","f",&o1,snap))
    seperr("write_snap: Cannot write o1 upon history file"); 
  float o2=dx*(first_cdp-pad-napL);
  if(0 != auxputch("o2","f",&o2,snap))
    seperr("write_snap: Cannot write o2 upon history file"); 
  float oSnap=0.0;
  if(0 != auxputch("o3","f",&oSnap,snap))
    seperr("write_snap: Cannot write o3 upon history file"); 

}

void Write::write_vshot() {
  // Save the velocity frame with random boundary conditions
  if(0 != auxputch("n1","d",&nkz,"vrandom"))
    seperr("write_vshot: Cannot write n1 upon history file"); 
  if(0 != auxputch("n2","d",&nkgx,"vrandom"))
    seperr("write_vshot: Cannot write n2 upon history file"); 
  if(0 != auxputch("d1","f",&dz,"vrandom"))
    seperr("write_vshot: Cannot write d1 upon history file"); 
  if(0 != auxputch("d2","f",&dx,"vrandom"))
    seperr("write_vshot: Cannot write d2 upon history file"); 
  float o1=dz*(-zpad-pad);
  if(0 != auxputch("o1","f",&o1,"vrandom"))
    seperr("write_vshot: Cannot write o1 upon history file"); 
  float o2=(-pad)*dx;
  if(0 != auxputch("o2","f",&o2,"vrandom"))
    seperr("write_vshot: Cannot write o2 upon history file"); 
}

void Write::write_amp(char* out_fn) {
  // Write amplitude information onto the history file
  if(0 != auxputch("rms_amplitude","f",&amp_rms,out_fn))
    seperr("write_amp: Cannot write rms_amplitude upon history file"); 
  if(0 != auxputch("mean_amplitude","f",&amp_mean,out_fn))
    seperr("write_amp: Cannot write mean_amplitude upon history file"); 
  if(0 != auxputch("min_amplitude","f",&amp_min,out_fn))
    seperr("write_amp: Cannot write min_amplitude upon history file"); 
  if(0 != auxputch("max__amplitude","f",&amp_max,out_fn))
    seperr("write_amp: Cannot write max_amplitude upon history file"); 
  if(0 != auxputch("nonzero_samples","f",&non_zero,out_fn))
    seperr("write_amp: Cannot write nonzero_samples upon history file"); 

  // Report amplitude and non-zero quantities
  std::cout << "************** CIC Stack section:*******" << std::endl;
  std::cout << "Non-zero samples: " << non_zero << std::endl;
  std::cout << "rms_amplitude: " << amp_rms << std::endl;
  std::cout << "mean_amplitude: " << amp_mean << std::endl;
  std::cout << "max_amplitude: " << amp_max << std::endl;
  std::cout << "min_amplitude: " << amp_min << std::endl;
  std::cout << "****************************************" << std::endl;
}

