#!/usr/bin/python
import copy
import os
import sepbase
import tempfile
import unittest
import re
import math
import sep_python as sep

from batch_task_executor import *
import pbs_util

### An example to define a batch task through inheriting the BatchTaskComposer class.
### IMPORTANT: Read /home/zyang03/script/batch_task_executor.py to learn 
### the detailed specifications/explanations of the interfaces defined in BatchTaskComposer.

class Psf2dBatchTaskComposer(BatchTaskComposer):
  def __init__(self, param_reader, njobs, nfiles_perjob):
    BatchTaskComposer.__init__(self)
    self.njobs = njobs
    self.nfiles_perjob = nfiles_perjob
    # Create all subjobs info
    self.subjobids = range(0, self.njobs)
    self.subjobfns_list = [None]*self.njobs
    path_out = param_reader.path_out
    args_psf = eq_args_from_cmdline
    # Create files for each subjob.
    for i in range(0,self.njobs):
       nfiles = self.nfiles_perjob
       self.subjobfns_list[i] = [None]*nfiles
       for j in range(0,nfiles):
          if i < 9:
            self.subjobfns_list[i][j] = '%s_00%s.H' % (args_psf['psf'], i+1)
          elif i < 99:
            self.subjobfns_list[i][j] = '%s_0%s.H'  % (args_psf['psf'], i+1)
          elif i < 999:
            self.subjobfns_list[i][j] = '%s_%s.H'   % (args_psf['psf'], i+1)
          print "subjobfns, %s" % (self.subjobfns_list[i][j])
    return

  def GetSubjobScripts(self, subjobid):
    '''Return the scripts content for that subjob.'''
    subjobfns = self.subjobfns_list[subjobid]
    scripts = []
    # Build scripts that create the files designated in subjobfns\
    jobname = subjobfns[0]
    shots = []
    shots = (re.findall('\d+', jobname ))
    shot_num = int(shots[0])
  #  sub_shot = int(shots[1])

    args_psf = eq_args_from_cmdline
    path_out = param_reader.path_out

    prefix = param_reader.prefix
    nshots = int(args_psf['nshots'])

    if(shot_num < 10):
      cmd =  '%s < %s wavelet_fn=%s par=%s extend=%s cur_shot=%s vel_frame=%s_00%s.H psf_born=1 ' \
            %(args_psf['bin'], args_psf['vel'], args_psf['wvt'], args_psf['par'], args_psf['extend'], \
              shot_num, args_psf['vel_frame_pref'], shot_num )
      cmd += '> %s_00%s psf=%s_00%s.H hess=%s_00%s.H vrandom=/dev/null \
                snap1=/dev/null snap2=/dev/null snap3=/dev/null ' \
               %(args_psf['log'],shot_num, args_psf['psf'], shot_num, args_psf['hess'], shot_num)
    elif(shot_num < 100):
      cmd =  '%s < %s wavelet_fn=%s par=%s extend=%s cur_shot=%s vel_frame=%s_0%s.H psf_born=1 ' \
             %(args_psf['bin'], args_psf['vel'], args_psf['wvt'], args_psf['par'], args_psf['extend'], \
               shot_num, args_psf['vel_frame_pref'], shot_num )
      cmd += '> %s_0%s psf=%s_0%s.H hess=%s_0%s.H vrandom=/dev/null \
                snap1=/dev/null snap2=/dev/null snap3=/dev/null ' \
               %(args_psf['log'],shot_num, args_psf['psf'], shot_num, args_psf['hess'], shot_num)
    elif(shot_num < 1000):
      cmd =  '%s < %s wavelet_fn=%s par=%s extend=%s cur_shot=%s vel_frame=%s_%s.H psf_born=1 ' \
             %(args_psf['bin'], args_psf['vel'], args_psf['wvt'], args_psf['par'], args_psf['extend'], \
               shot_num, args_psf['vel_frame_pref'], shot_num )
      cmd += '> %s_%s psf=%s_%s.H hess=%s_%s.H vrandom=/dev/null \
                snap1=/dev/null snap2=/dev/null snap3=/dev/null ' \
               %(args_psf['log'],shot_num, args_psf['psf'], shot_num, args_psf['hess'], shot_num)
    else:   
      print "Incorrect shot_num: %s" % shot_num 

    scripts.append(cmd)
    return scripts, str(subjobid)


  def GetSubjobsInfo(self):
    '''Should return two lists, subjobids and subjobfns.'''
    return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)


if __name__ == '__main__':
  print "Run BatchTaskExecutor_test with params:", sys.argv
  # Check basic environment variable setup.
  assert 'SEP' in os.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
  assert 'RSF' in os.environ, '!Environment var RSF is not set yet, should have your RSF environment set up first! Check paths like /opt/RSF'
  eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
  #dict_args = sepbase.RetrieveAllEqArgs(eq_args_from_cmdline)
  param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)
  njobs=int(eq_args_from_cmdline['nshots'])
  nfiles_perjob=1
  psf2d_btc = Psf2dBatchTaskComposer(param_reader,njobs,nfiles_perjob)
  bte = BatchTaskExecutor(param_reader)
  prefix = param_reader.prefix

  print "Launch the thing..."
  bte.LaunchBatchTask(prefix, psf2d_btc)

  # Combine the files to one single file.
  _, fns_list = psf2d_btc.GetSubjobsInfo()
  fn_seph_list = [fn for fns in fns_list for fn in fns]
  args_psf = eq_args_from_cmdline
  stack_fn='%s_stack.H'   % (args_psf['psf'])
  CombineMultipleOutputSephFiles(bte, fn_seph_list, stack_fn)
  
  sepbin=os.environ["SEP"]+"/bin"
  stat1,nzc=sep.RunShellCmd("%s/Get parform=no <%s_001.H nzc"%(sepbin,args_psf['psf']))
  print "nzc: " + nzc
  stat1,nxc=sep.RunShellCmd("%s/Get parform=no <%s_001.H nxc"%(sepbin,args_psf['psf']))
  print "nxc: " + nxc
  cmd="echo 'nzc=%s' >> %s"%(nzc,stack_fn)
  os.system(cmd)
  cmd="echo 'nxc=%s' >> %s"%(nxc,stack_fn)
  os.system(cmd)


#  stack_fn2 = '%s_stack.H'   % (args_psf['psf_reg'])
#  fn_seph_list2 = [fn.replace('_psf', '_psf_reg') for fns in fns_list for fn in fns]
#  CombineMultipleOutputSephFiles(bte, fn_seph_list2, stack_fn2)
#
#  stack_fn3 = '%s/%s_psf_holes_stack.H'   % (args_psf['resdir'], args_psf['projpref'])
#  fn_seph_list3 = [fn.replace('_psf', '_psf_holes') for fns in fns_list for fn in fns]
#  CombineMultipleOutputSephFiles(bte, fn_seph_list3, stack_fn3)

  # Check the final result
  #stat,out = commands.getstatusoutput('Attr < test-all.H param=1 | Get parform=n totsamp ')
  #assert stat==0 and int(out)==4, 'Test NOT successful, final output test-all.H is not valid!'
  print 'GODDAMMIT, WE MADE IT.'
